﻿'┌────┐
'│デザイン├──────────────────────
'└┬───┘
'  │『レイアウトについて』
'  │・システムメニューから裏メニューを開けるときに、メニュー選択後ダイヤルボタンへマウスカーソルを持って行くのが大変にならないように、なるべく小さくしたい。
'  │・ダイヤルボタンは強制的にフォーカスを奪い、右利きではマウスカーソルを右にずらす場合が多いので、合言葉入力欄より左にダイヤルを置くことにした。
'  │・良くある暗証番号は4桁であり、余裕を持ってその2倍ぐらいあればいいだろうということで、合言葉入力欄は8文字分を最低サイズとした。
'  │・[OK]ボタンは、ダイヤルボタンに近い方がマウス操作が楽なので、ダイヤルボタンと合言葉欄の間付近に固定し、右の余白を[キャンセル]ボタンで埋めることにした。
'  └────────────



''' <summary>
''' 鍵入力フォーム
''' </summary>
''' <remarks></remarks>
Public Class FormKey

    ''' <summary>正しいダイヤル</summary>
    Private ReadOnly CorrectDial As String = KeySettings.Dial

    ''' <summary>正しい合言葉</summary>
    Private ReadOnly CorrectPassword As String = KeySettings.Password

    '備忘録・・・《ReadOnly》付き変数は、宣言時かコンストラクタ（New()）内で代入できる。
    '            "Public Sub New() ～ End Sub"は、このコードエディタ上部のクラス名ボックス「FormKey」/メソッド名ボックス「New()」を選択すれば、自動挿入してくれる。


    ''' <summary>詳細エラーモード</summary>
    Private DetailMode As Boolean = False



    '============================================================================================




    ''' <summary>
    ''' コンストラクタ
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub New()

        '=== 初期化 ===
        MyBase.New() '←要るのかな？ わかんないんで一応やっとく。害もなさそうやし。
        InitializeComponent() 'この呼び出しはデザイナーで必要です。

        '--- ダイヤルボタン ---
        With dbtnDial
            .SlipWheel = SystemSettings.SecretMenu.MouseWheel.SlipWheelMode
            .WheelDelta = SystemSettings.SecretMenu.MouseWheel.Delta
            .RollSeparateTime = SystemSettings.SecretMenu.MouseWheel.RollSeparateTime
        End With

    End Sub

    ''' <summary>
    ''' コンストラクタ
    ''' </summary>
    ''' <param name="Message">ダイアログに表示するメッセージを指定します。省略または空文字なら、デフォルトの文章を使用します。</param>
    ''' <param name="Dial">使用するダイヤルパターンを指定します。</param>
    ''' <param name="Password">使用する合言葉を指定します。</param>
    ''' <param name="DetailError">詳細エラーモードを有効にするかを指定します。</param>
    ''' <remarks></remarks>
    Public Sub New(Message As String, Optional Dial As String = Nothing, Optional Password As String = Nothing, Optional DetailError As Boolean = False)

        '=== 初期化 ===
        Me.New()

        '=== 設定 ===
        If Not IsNothing(Dial) Then CorrectDial = Dial
        If Not IsNothing(Password) Then CorrectPassword = Password
        DetailMode = DetailError

        '--- メッセージに合わせてサイズ調整 ---
        '《lblMessage》をデザイン時に標準状態でのサイズにしておく。《AutoSize》プロパティも《False》にしておく。
        'メッセージ変更前の《lblMessage》から変更（《AutoSize》プロパティを《True》しにて《Text》プロパティを設定）後のサイズを引く。
        'その結果、標準状態のサイズより大きくなっているなら、溢れた分だけフォームを大きくする。

        Dim AdjustSize As Size = lblMessage.Size 'Formサイズ調整用

        lblMessage.AutoSize = True
        lblMessage.Text = Message

        AdjustSize = lblMessage.Size - AdjustSize

        If AdjustSize.Height > 0 Then
            TableLayoutPanel1.Anchor = TableLayoutPanel1.Anchor Xor AnchorStyles.Top

            Me.Height += AdjustSize.Height

            TableLayoutPanel1.Anchor = TableLayoutPanel1.Anchor Xor AnchorStyles.Top
        Else
            AdjustSize.Height = 0
        End If

        If AdjustSize.Width > 0 Then
            Me.Width += AdjustSize.Width
        Else
            AdjustSize.Width = 0
        End If

        Me.MinimumSize += AdjustSize

    End Sub

    ''' <summary>
    ''' 一時的な無段ホイール モードを指定して、モーダルフォームを表示する。
    ''' </summary>
    ''' <param name="Owner">所有者</param>
    ''' <param name="SlipWheelMode">無段ホイール モード</param>
    ''' <remarks></remarks>
    Public Overloads Function ShowDialog(Owner As System.Windows.Forms.IWin32Window, SlipWheelMode As Boolean) As System.Windows.Forms.DialogResult

        dbtnDial.SlipWheel = SlipWheelMode
        Return MyBase.ShowDialog(Owner)

    End Function

    Private Sub FormKey_Shown(sender As Object, e As System.EventArgs) Handles Me.Shown

        '=== 最大サイズを適当な値に設定 ===
        'Me.MaximumSize = New Size(Me.Width * 3, Me.Height * 1.5)

    End Sub

    Private Sub btnOK_Click(sender As Object, e As System.EventArgs) Handles btnOK.Click

        '=== 鍵の照合 ===
        Dim dialError As Boolean = (dbtnDial.StepPattern <> CorrectDial) 'ダイヤル
        Dim passwordError As Boolean = (txtPassword.Text <> CorrectPassword) '合言葉

        '=== 警告 ===
        If dialError OrElse passwordError Then

            erpErrorMessage.Clear()

            If DetailMode Then

                '詳細エラーモード
                If dialError Then erpErrorMessage.SetError(dbtnDial, "ダイヤルが違います。")
                If passwordError Then erpErrorMessage.SetError(txtPassword, "合言葉が違います。")

            Else

                '簡易エラーモード
                dialError = True
                passwordError = True

                erpErrorMessage.SetIconAlignment(btnOK, ErrorIconAlignment.MiddleLeft)
                erpErrorMessage.SetError(btnOK, "鍵が違います。" & vbCrLf & "訂正後、再度クリックしてください。")

            End If

            My.Computer.Audio.PlaySystemSound(Media.SystemSounds.Exclamation)

            '--- 修正箇所を消去 ---（ついでに、フォーカスも移動）
            If dialError Then dbtnDial.StepPattern = ""
            If passwordError Then txtPassword.Text = ""
            txtPassword.Focus()

            '--- リザルト変更 ---
            Me.DialogResult = Windows.Forms.DialogResult.None 'フォームが勝手に閉じるのを防ぐ（各ボタンの《.DialogResult》プロパティを設定しているため、自動的に閉じられる）。

        End If

    End Sub

    Private Sub btnHelp_Click(sender As System.Object, e As System.EventArgs) Handles btnHelp.Click

        '=== 説明文 ===
        Help.ShowPopup(btnHelp,
                       "鍵（ダイヤルと合言葉）を入力して、[OK]をクリックします。" & vbCrLf &
                       "使用していない側は、未入力または空欄のままにしておきます。" & vbCrLf & vbCrLf &
                       "鍵が一致しなかった場合は、ダイヤルまたは合言葉の右側か[OK]ボタンの左側の何れかにエラーが表示されます。" & vbCrLf &
                       "入力し直して、再度[OK]をクリックしてください。",
                       Control.MousePosition)

    End Sub

End Class