﻿'メインの一部（ランチメニューの構築と実行）

'●コードの分割方法。
'ソリューションエクスプローラーのプロジェクト名を右クリック→[追加]→[クラス]→[コードファイル]→クラス名に[Partial]を付けて宣言



Partial Public Class FormMain

    ''' <summary>根元フォルダ展開アイテムの目印</summary>
    Private Const ROOT_DIRECTORY_MARK As String = "!RootDirectory!"

    ''' <summary>枝葉根元フォルダ展開アイテムの目印</summary>
    Private Const SUB_DIRECTORY_MARK As String = "!SubDirectory!"

    ''' <summary>未到アイテムの目印</summary>
    Private Const UNREACHED_MARK As String = "!Unreached!"

    ''' <summary>不明アイテムの目印</summary>
    Private Const MISSING_MARK As String = "!Missing!"


    ''' <summary>手動再構築メニュー項目の名前</summary>
    Private Const HAND_REBUILD_NAME As String = "tsmReBuild"

    ''' <summary>操作アイテムの区切り線の名前</summary>
    Private Const OPERATION_ITEM_NAME As String = "tssOperationSeparator"

    ''' <summary>ランチメニュー登録フォルダの展開メニュー項目の名前</summary>
    Private Const DIRECTORY_OPEN_NAME As String = "tsmDirectoryOpen"

    ''' <summary>一括実行メニュー項目の名前</summary>
    Private Const BATCH_RUN_NAME As String = "tsmBatchRun"


    ''' <summary>構築中エラーメッセージ</summary>
    Private Const BUILDING_ERROR_MESSAGE As String = "ランチメニューの構築中に例外が発生したため、中断しました。{0}原因に対処後、ランチメニューの[更新]を選択し開き直してください。{0}※構築中は、ランチメニュー登録フォルダを操作しないでください。{0}{0}『{1}』"


    '--------------------------------------------------------------------------------------------


    ''' <summary>格納アイテム用ダミー画像</summary>
    Dim DirectoryItemDummyIcon As New Bitmap(1, 1) 'サイズはどうでも良い。透明なので何も表示されないのと同じ。


    ''' <summary>押されたマウスのボタン</summary>
    Dim PushedButton As MouseButtons = Windows.Forms.MouseButtons.None



    '============================================================================================



    ''' <summary>
    ''' ランチメニューを根っこから構築
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub LaunchMenuRootBuild()

        LaunchMenuAllNew.Value = False

        With ctxLaunchMenu.Items

            '=== 必須項目の作成 ===
            '--- [裏メニュー]項目の退避 ---
            Dim temp(1) As ToolStripItem
            If SystemSettings.SecretMenu.Locate = SystemParameter.SecretMenuParameter.LocateNo.LaunchMenu Then
                temp(0) = tssSecretMenu
                temp(1) = tsmSecretMenu
            End If

            '--- 全消去 ---
            .Clear()

            '--- 常設項目の設置 ---
            .Add(New ToolStripMenuItem("閉じる(&C)") With {.Visible = SystemSettings.LaunchMenu.HandClose, .ToolTipText = "ランチメニューを閉じます。"})
            .Add(New ToolStripMenuItem("再構築(&R)", Nothing, New EventHandler(AddressOf LaunchMenuReBuild_Click)) With {.Name = HAND_REBUILD_NAME, .Visible = False, .ToolTipText = "ランチメニューを再構築します。", .Tag = False}) '《Tag》をこの項目の固定表示のフラグとして使用する。
            .AddRange(NewOperationItem(ROOT_DIRECTORY_MARK))

            '--- [裏メニュー]項目の設置 ---
            If SystemSettings.SecretMenu.Locate = SystemParameter.SecretMenuParameter.LocateNo.LaunchMenu Then .AddRange(temp)

            '--- 区切りの追加 ---
            .Add(New ToolStripSeparator)

            '=== ランチメニュー作成 ===
            If System.IO.Directory.Exists(SystemSettings.LaunchMenu.RegistryPath) Then

                '--- アイテム構築 ---
                LaunchMenuLayerBuild(ctxLaunchMenu, 0)

                '--- ランチメニュー登録フォルダの変更監視 ---
                With fswRegistryItemWatcher
                    If .Path <> SystemSettings.LaunchMenu.RegistryPath Then
                        .Path = SystemSettings.LaunchMenu.RegistryPath
                    End If
                    .EnableRaisingEvents = True
                End With

            Else

                '--- ランチメニュー登録フォルダが存在しない ---
                .Add(New ToolStripMenuItem("（ランチメニュー登録フォルダ所在不明）", SystemIcons.Warning.ToBitmap) With {.Enabled = False, .Tag = MISSING_MARK, .ToolTipText = "設定を見直してください。"})
                HoldHandRebuildItem()

            End If

        End With

    End Sub

    ''' <summary>
    ''' ランチメニューの各層を構築
    ''' </summary>
    ''' <param name="ParentItem">親となる格納アイテム</param>
    ''' <param name="DepthCounter">構築深度カウンタ</param>
    ''' <remarks></remarks>
    Private Overloads Sub LaunchMenuLayerBuild(ParentItem As ContextMenuStrip, DepthCounter As Integer)

        '=== メニューを構築（メニューの根元用） ===
        With ParentItem

            Try

                'Throw New IO.IOException 'テスト：擬似的にエラーを発生してみる。

                '--- 格納アイテム（フォルダ）を登録する ---
                For Each dItem As String In System.IO.Directory.GetDirectories(SystemSettings.LaunchMenu.RegistryPath) '全てのサブフォルダを取得

                    'System.Threading.Thread.Sleep(1000) 'テスト

                    Dim newItem As ToolStripMenuItem = NewDirectoryItem(dItem)
                    If newItem IsNot Nothing Then
                        .Items.Add(newItem) '登録

                        '一括構築深度チェック
                        If Not BuildDepthChecker(DepthCounter) Then LaunchMenuLayerBuild(newItem, DepthCounter + 1) 'さらに下の階層へ
                    End If

                    My.Application.DoEvents()

                Next

                '--- 実行アイテム（ファイル）を登録する ---
                For Each fItem As String In System.IO.Directory.GetFiles(SystemSettings.LaunchMenu.RegistryPath) '全てのサブファイルを取得

                    'System.Threading.Thread.Sleep(1000) 'テスト

                    Dim newItem As ToolStripMenuItem = NewFileItem(fItem)
                    If newItem IsNot Nothing Then .Items.Add(newItem) '登録

                    My.Application.DoEvents()

                Next

            Catch ex As Exception

                MessageBox.Show(String.Format(BUILDING_ERROR_MESSAGE, vbCrLf, ex.Message), ntfTrayIcon.Text, MessageBoxButtons.OK, MessageBoxIcon.Error)
                HoldHandRebuildItem()

            End Try

            '--- 空格納アイテムのチェック ---
            If .Items(.Items.Count - 1).GetType Is GetType(ToolStripSeparator) Then 'メニューの最後尾が区切り線なら、追加がなかったということ
                .Items.Add(NewEmptyItem)
            End If

        End With

    End Sub

    ''' <summary>
    ''' ランチメニューの各層を構築
    ''' </summary>
    ''' <param name="ParentItem">親となる格納アイテム</param>
    ''' <param name="DepthCounter">構築深度カウンタ</param>
    ''' <remarks></remarks>
    Private Overloads Sub LaunchMenuLayerBuild(ParentItem As ToolStripMenuItem, DepthCounter As Integer)

        '=== メニューを構築（メニューの枝葉用） ===
        With ParentItem

            ''--- 格納アイテム（フォルダ）の存在確認 ---（めったに起こらない現象であるだけでなく、ここでは対象となる時間が短すぎ、ほとんど下のTry文内で捉えられることになると思うので、省略）
            'If Not System.IO.Directory.Exists(.Tag) Then
            '    '見失ったら、強制的にランチメニューを閉じて再構築する
            '    ctxLaunchMenu.Close()
            '    LaunchMenuAllNew.Value = True
            '    ntfTrayIcon.ShowBalloonTip(10000, "例外", "ランチメニュー構築時に、対象フォルダを見失いました。" & vbCrLf & "構築中は、ランチメニュー登録フォルダを操作しないでください。", ToolTipIcon.Error)
            '    Exit Sub '※※※
            'End If

            '--- 準備 ---
            .DropDownItems.Clear()

            Try

                '--- 格納アイテム（フォルダ）を登録する ---
                For Each dItem As String In System.IO.Directory.GetDirectories(.Tag) '全てのサブフォルダを取得

                    'System.Threading.Thread.Sleep(1000) 'テスト

                    Dim newItem As ToolStripMenuItem = NewDirectoryItem(dItem)
                    If newItem IsNot Nothing Then
                        .DropDownItems.Add(newItem) '登録

                        If Not BuildDepthChecker(DepthCounter) Then LaunchMenuLayerBuild(newItem, DepthCounter + 1) 'さらに下の階層へ
                    End If

                    My.Application.DoEvents()

                Next

                '--- 実行アイテム（ファイル）を登録する ---
                For Each fItem As String In System.IO.Directory.GetFiles(.Tag) '全てのサブファイルを取得

                    'System.Threading.Thread.Sleep(1000) 'テスト

                    Dim newItem As ToolStripMenuItem = NewFileItem(fItem)
                    If newItem IsNot Nothing Then .DropDownItems.Add(newItem) '登録

                    My.Application.DoEvents()

                Next

            Catch ex As Exception

                MessageBox.Show(String.Format(BUILDING_ERROR_MESSAGE, vbCrLf, ex.Message), ntfTrayIcon.Text, MessageBoxButtons.OK, MessageBoxIcon.Error)
                HoldHandRebuildItem()

            End Try

            '--- 空格納アイテムのチェック ---
            If .DropDownItems.Count = 0 Then .DropDownItems.Add(NewEmptyItem) '追加がない

            '--- 常設項目の設置 ---
            .DropDownItems.Add(New ToolStripSeparator With {.Name = OPERATION_ITEM_NAME})
            .DropDownItems.AddRange(NewOperationItem(SUB_DIRECTORY_MARK))

            '--- 親の後始末 ---
            .ForeColor = SystemColors.ControlText

        End With

    End Sub

    ''' <summary>
    ''' 現在の構築深度チェック
    ''' </summary>
    ''' <param name="DepthCounter">深度カウンタ</param>
    ''' <returns>《True》 = 制限値到達 / 《False》 = 制限値未達</returns>
    ''' <remarks></remarks>
    Private Function BuildDepthChecker(DepthCounter As Integer) As Boolean

        Return (DepthCounter >= SystemSettings.LaunchMenu.MenuBuild.Depth)

    End Function

    ''' <summary>
    ''' 手動更新項目の固定化
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub HoldHandRebuildItem()

        With ctxLaunchMenu.Items(HAND_REBUILD_NAME) '再構築に成功したら自動的に非表示となる。
            .ForeColor = Color.Red
            .Visible = True
            .Tag = True '手動更新項目を固定するフラグ。
        End With

    End Sub

    ''' <summary>
    ''' 新しい操作アイテムを作成する
    ''' </summary>
    ''' <param name="DirectoryMark">アプリケーション登録フォルダのマーク</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Private Function NewOperationItem(DirectoryMark As String) As ToolStripMenuItem()

        Dim newItems As ToolStripMenuItem() = {New ToolStripMenuItem("登録・削除(&O)", Nothing, New EventHandler(AddressOf OpenRegistryDirectory_Click)) With {.Name = DIRECTORY_OPEN_NAME,
                                                                                                                                                               .ToolTipText = IIf(DirectoryMark = ROOT_DIRECTORY_MARK, "ランチメニュー", "この層に対応する") & "登録フォルダを開きます。",
                                                                                                                                                               .Tag = DirectoryMark},
                                               New ToolStripMenuItem("一括実行(&B)", Nothing, New EventHandler(AddressOf BatchRun_Click)) With {.Name = BATCH_RUN_NAME,
                                                                                                                                                .ToolTipText = "この層に登録されている実行アイテムを全て実行します。",
                                                                                                                                                .Tag = DirectoryMark}}

        Return newItems

    End Function

    ''' <summary>
    ''' 新しい格納アイテムを作成する
    ''' </summary>
    ''' <param name="Path">ディレクトリのパス</param>
    ''' <returns>新アイテム / 《Nothing》 = 施錠された裏アイテム</returns>
    ''' <remarks></remarks>
    Private Function NewDirectoryItem(Path As String) As ToolStripMenuItem

        Try

            Dim newItem As ToolStripMenuItem = Nothing

            If UnLocked.Value OrElse (Not (New System.IO.DirectoryInfo(Path).Attributes And IO.FileAttributes.Hidden) = IO.FileAttributes.Hidden) Then '裏アイテム（不可視属性フォルダ）は、裏メニューが開いていない場合には表示しない。

                Dim itemName As String = System.IO.Path.GetFileName(Path)

                newItem = New ToolStripMenuItem(itemName, DirectoryItemDummyIcon) With {.Name = itemName, .Tag = Path, .BackColor = SystemSettings.LaunchMenu.DirectoryItemColor.Color} '新しいアイテムを作成する。変更時の検索用に名前も設定する。大きいアイコン時の構築中マーク表示スペースを確保しておくため、ダミーの画像を設定しておく。
                SetUnreachedTag(newItem) '未到格納アイテムであることを示す。
                AddHandler newItem.DropDown.MouseEnter, AddressOf ctxLaunchMenu_MouseEnter 'メニュー上にマウスが入ったことを知る（自動撤収機能のコントロール用）。
                AddHandler newItem.DropDown.MouseLeave, AddressOf ctxLaunchMenu_MouseLeave 'メニュー上からマウスが出たことを知る（自動撤収機能のコントロール用）。
                AddHandler newItem.DropDown.PreviewKeyDown, AddressOf ctxLaunchMenu_PreviewKeyDown 'メニューをキー操作したことを知る（自動撤収機能のコントロール用）。
                AddHandler newItem.DropDownOpening, AddressOf MenuDirectoryItem_DropDownOpening 'ドロップダウンメニューが展開されることを知る。（《newItem.DropDown.Opening》イベントだと開くのをキャンセルする機能があるけど、上手く行かない。）
                AddHandler newItem.DropDownOpened, AddressOf MenuDirectoryItem_DropDownOpened 'ドロップダウンメニューが表示されたことを知る。

            End If

            Return newItem

        Catch ex As Exception

            Throw 'エラーを再スロー

        End Try

    End Function

    ''' <summary>
    ''' メニューアイテムに「未到」印をつける
    ''' </summary>
    ''' <param name="Item">対象アイテム</param>
    ''' <remarks></remarks>
    Private Sub SetUnreachedTag(Item As ToolStripMenuItem)

        With Item
            .ForeColor = SystemColors.GrayText '未到格納アイテムの前景色は灰色。
            '.Image = My.Resources.modify.ToBitmap '要構築印（“大きいアイコン”時に恰好悪いので止める）。
            .DropDownItems.Insert(0, NewUnreachedItem) '未到格納アイテムであることを示すサブアイテムを先頭に付ける。
        End With

    End Sub

    ''' <summary>
    ''' 新しい実行アイテムを作成する
    ''' </summary>
    ''' <param name="Path">ファイルのパス</param>
    ''' <returns>新アイテム / 《Nothing》 = 施錠された裏アイテム</returns>
    ''' <remarks></remarks>
    Private Function NewFileItem(Path As String) As ToolStripMenuItem

        Try

            Dim newItem As ToolStripMenuItem = Nothing

            If UnLocked.Value OrElse (Not (System.IO.File.GetAttributes(Path) And IO.FileAttributes.Hidden) = IO.FileAttributes.Hidden) Then '裏アイテム（不可視属性ファイル）は、裏メニューが開いていない場合には表示しない。

                Dim fName As String 'メニューに表示する名前（ファイル名）
                If SystemSettings.LaunchMenu.ShowExtension Then
                    fName = System.IO.Path.GetFileName(Path) '拡張子あり
                Else
                    fName = System.IO.Path.GetFileNameWithoutExtension(Path) '拡張子なし
                End If

                newItem = New ToolStripMenuItem(fName,
                                                System.Drawing.Icon.ExtractAssociatedIcon(Path).ToBitmap,
                                                New EventHandler(AddressOf MenuFileItem_Click)) With {.Tag = Path} '新しいアイテムを作成する
                AddHandler newItem.MouseDown, AddressOf MenuItem_MouseDown 'マウスのどのボタンでクリックしたかを知るため

            End If

            Return newItem

        Catch ex As Exception

            Throw 'エラーを再スロー

        End Try

    End Function

    ''' <summary>
    ''' 新しい未到アイテムを作成する
    ''' </summary>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Private Function NewUnreachedItem() As ToolStripMenuItem

        Return New ToolStripMenuItem("（未構築）", SystemIcons.Information.ToBitmap) With {.Enabled = False,
                                                                                           .Tag = UNREACHED_MARK,
                                                                                           .ToolTipText = "他を構築していませんか？" & vbCrLf & "終了を待って、選択し直してください。"}

    End Function

    ''' <summary>
    ''' 新しい空アイテムを作成する
    ''' </summary>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Private Function NewEmptyItem() As ToolStripMenuItem

        Return New ToolStripMenuItem("（空）", SystemIcons.Information.ToBitmap) With {.Enabled = False, .ToolTipText = "何も登録されていません。"}

    End Function

    Private Sub MenuItem_MouseDown(sender As System.Object, e As MouseEventArgs)

        '=== アイテムをクリックしたマウスのボタンを記録 ===
        PushedButton = e.Button

    End Sub

    Private Sub MenuFileItem_Click(sender As System.Object, e As System.EventArgs)

        '=== メニューの実行アイテムがクリックされた ===
        '--- コマンドライン引数を準備（右クリック時用） ---
        Dim argumentsList As String = "" 'コマンドライン引数

        If PushedButton = Windows.Forms.MouseButtons.Right Then

            'クリップボードの内容を取得
            If Clipboard.ContainsFileDropList Then

                For Each fileName As String In Clipboard.GetFileDropList '何時もと違うアプリでファイルを開く時など
                    argumentsList &= " """ & fileName & """"
                Next
                'argumentsList = argumentsList.Trim() '前後に余計な空白があっても、適切に処理されるみたい。

            ElseIf Clipboard.ContainsText Then

                argumentsList = Clipboard.GetText 'Webブラウザにアドレスを渡す時など（空白文字を含める場合は、自前でダブルクォーテーションで括らないといけない）

            End If

            '調整
            If My.Computer.Keyboard.CtrlKeyDown Xor SystemSettings.LaunchMenu.ExCommandLineOption Then '[Ctrl]キーと《常用フラグ》の排他的論理和をとることで、拡張コマンドライン引数モードの使用を反転させられる。
                argumentsList = InputBox(String.Format("コマンドライン引数を調整してください。{0}{0}{0}実行アイテム：{0}{0}{1}", vbCrLf, DirectCast(sender, ToolStripMenuItem).Tag),
                                         Me.ntfTrayIcon.Text & "- [拡張コマンドライン引数]",
                                         argumentsList)

                If argumentsList Is Environ(255) Then Exit Sub 'よく分からんけど、《InputBox》で[キャンセル]すると、戻り値が《Environ(255)》になるみたい。これで、""で[OK]した場合と区別できる。ただし、《argumentsList》の宣言時の空文字とは区別できないみたい。
            End If

        End If

        '--- 実行する ---
        ItemRun(DirectCast(sender, ToolStripMenuItem).Tag, argumentsList)

    End Sub

    Private Sub OpenRegistryDirectory_Click(sender As System.Object, e As System.EventArgs)

        '=== ランチメニュー登録フォルダを開く ===
        With DirectCast(sender, ToolStripMenuItem)

            '--- フォルダのパスを確定 ---
            Dim path As String = ""
            If DirectCast(sender, ToolStripMenuItem).Tag = SUB_DIRECTORY_MARK Then
                path = .OwnerItem.Tag
            Else
                path = SystemSettings.LaunchMenu.RegistryPath
            End If

            '--- 開く ---
            If System.IO.Directory.Exists(path) Then

                Try

                    System.Diagnostics.Process.Start(path)

                Catch ex As Exception

                    MessageBox.Show(String.Format("ランチメニュー登録フォルダが開けませんでした。{0}『{1}』{0}●場所{0}{2}", vbCrLf & vbCrLf, ex.Message, path),
                                    ntfTrayIcon.Text,
                                    MessageBoxButtons.OK,
                                    MessageBoxIcon.Error)

                End Try

            Else

                MessageBox.Show(String.Format("ランチメニュー登録フォルダが、見つかりませんでした。{0}設定や登録内容を見直してください。{0}{0}●場所{0}{1}", vbCrLf, path),
                                ntfTrayIcon.Text,
                                MessageBoxButtons.OK,
                                MessageBoxIcon.Error)
                LaunchMenuAllNew.Value = True

            End If

        End With

    End Sub

    Private Sub BatchRun_Click(sender As System.Object, e As System.EventArgs)

        '=== 同階層の実行アイテムを一括（順次）実行する ===
        '--- 対象メニューを確定 ---
        Dim target As ToolStripItemCollection
        If DirectCast(sender, ToolStripMenuItem).Tag = ROOT_DIRECTORY_MARK Then
            target = ctxLaunchMenu.Items
        Else
            target = DirectCast(DirectCast(sender, ToolStripMenuItem).OwnerItem, ToolStripMenuItem).DropDownItems
        End If

        '--- 実行 ---
        For Each item As ToolStripItem In target

            Select Case item.Tag

                Case ROOT_DIRECTORY_MARK, SUB_DIRECTORY_MARK, Nothing

                    Exit Select '※※特殊タグ・無タグの物は、実行アイテムではない。

                Case Else

                    If DirectCast(item, ToolStripMenuItem).DropDownItems.Count = 0 Then '子アイテムを持っていない物、すなわち実行アイテム
                        ItemRun(item.Tag)
                        My.Application.DoEvents()
                        'Threading.Thread.Sleep(5000) 'インターバルを挟んでも良いけど、起動時間はアプリによってまちまちなんで、あんまり意味ないかも。5秒は、Norton Internet Security 2012の起動マネージャのマネ。
                    End If

            End Select

        Next

        'ショートカットを実行した場合、《p》にプロセスが返されないので、リンク先を取得してexeファイルだったら起動する。
        'この方法だと、起動待ちはできるが、ショートカットに設定したコマンドラインオプションが反映できない。
        'Dim p As System.Diagnostics.Process
        'For Each item As ToolStripDropDownItem In DirectCast(sender, ToolStripMenuItem).DropDownItems

        '    Dim target As String = ReturnExeFilePath(item.Tag)
        '    If target <> "" AndAlso System.IO.File.Exists(target) Then
        '        p = System.Diagnostics.Process.Start(target)
        '        p.WaitForInputIdle()
        '    End If

        'Next

    End Sub

    Private Sub LaunchMenuReBuild_Click(sender As System.Object, e As System.EventArgs)

        '=== ランチメニューの強制再構築 ===
        LaunchMenuAllNew.Value = True

    End Sub

    ''' <summary>
    ''' 実行アイテムを実行する
    ''' </summary>
    ''' <param name="FileName">アイテムのパスとファイル名</param>
    ''' <param name="Arguments">コマンドライン引数</param>
    ''' <remarks></remarks>
    Private Sub ItemRun(FileName As String, Optional Arguments As String = "")

        Try

            'Throw New System.IO.FileNotFoundException 'テスト：擬似的にエラーを発生してみる。

            System.Diagnostics.Process.Start(FileName, Arguments)

            'Win32 APIの《ShellExecuteEX》では、動詞に"properties"を指定することで「ファイルのプロパティ」ダイアログを表示できた。(http://hp.vector.co.jp/authors/VA024411/vbtips02.html)
            'なので、《Process.Start》メソッドでも同じようにできるかと期待したけど駄目だった。このメソッドの正体は、《ShellExecute》APIのようだ。
            '当初は、ランチメニューからプロパティ画面の表示をもくろんだが、「登録内容の設定は登録フォルダ内（エクスプローラ）でやる」と役割分担を明確にすることにして、諦めた。

        Catch ex As System.IO.FileNotFoundException

            MessageBox.Show(String.Format("当該ファイルが、存在しません。{0}{0}●場所{0}{1}", vbCrLf, FileName),
                           ntfTrayIcon.Text,
                           MessageBoxButtons.OK,
                           MessageBoxIcon.Error)
            LaunchMenuAllNew.Value = True

            'Try構文でエラーを捉えるには、多くのリソースを消費するから、Try構文の前に《System.IO.File.Exists(～)》でファイルの存在確認を済ませておくのが良いと読んだことがある。
            'ここでは、ファイルが存在しないというのは稀な事態でそれ程重要でもないと思うので、Try内で一緒に対処することにした。

        Catch ex As Exception

            MessageBox.Show(String.Format("実行アイテムの実行に失敗しました。{0}{0}『{1}』{0}{0}●場所{0}{2}{0}{0}●コマンドライン引数{0}{3}", vbCrLf, ex.Message, FileName, Arguments),
                            ntfTrayIcon.Text,
                            MessageBoxButtons.OK,
                            MessageBoxIcon.Error)

        End Try

    End Sub

    ' ''' <summary>
    ' ''' ショートカットアイコンのリンク先が、exeファイルだったらそのパスを返す（使ってません・・・）
    ' ''' </summary>
    ' ''' <param name="Path">調べたいパス</param>
    ' ''' <returns>exeファイルならそのパスを、違ったら空文字を返す。</returns>
    ' ''' <remarks></remarks>
    'Private Function ReturnExeFilePath(Path As String) As String

    '    Dim ret As String = ""

    '    If System.IO.Path.GetExtension(Path).Equals(".exe", StringComparison.OrdinalIgnoreCase) Then

    '        ret = Path

    '    ElseIf System.IO.Path.GetExtension(Path).Equals(".lnk", StringComparison.OrdinalIgnoreCase) Then

    '        'WSHでショートカットアイコンのリンク先を取得する（ショートカット作成方法からの派生）。
    '        Dim oShell As Object = CreateObject("WScript.Shell")
    '        Dim oShortcut As Object = oShell.CreateShortcut(Path)
    '        Dim linkTarget As String = oShortcut.TargetPath

    '        If System.IO.Path.GetExtension(linkTarget).Equals(".exe", StringComparison.OrdinalIgnoreCase) Then
    '            ret = linkTarget '& " " & oShortcut.Arguments'←これを付けるとコマンドラインオプションも返せるが、その文字列からファイルのパスを取得できずファイルの存在チェックなどができなくなる。
    '        End If

    '    End If

    '    Return ret

    'End Function

End Class