﻿'メインの一部（ランチメニューの展開と撤収）

'●コードの分割方法。
'ソリューションエクスプローラーのプロジェクト名を右クリック→[追加]→[クラス]→[コードファイル]→クラス名に[Partial]を付けて宣言



Partial Public Class FormMain

    ''' <summary>
    ''' ゲージパラメータ
    ''' </summary>
    ''' <remarks></remarks>
    Private Structure GaugeParameter

        ''' <summary>
        ''' 形式種（ホバー継続時間ゲージ）
        ''' </summary>
        ''' <remarks></remarks>
        Public Enum GaugeStyleType As Byte

            ''' <summary>横向き</summary>
            Horizontal

            ''' <summary>縦向き</summary>
            Vertical

        End Enum


        '----------------------------------------------------------------------------------------------------------------------------


        ''' <summary>形式</summary>
        Public Property Style As GaugeStyleType

        ''' <summary>形状</summary>
        Public Property BaseRect As Rectangle

        ''' <summary>色</summary>
        Public Property BaseColor As Color

        ''' <summary>影の濃さ（透過量）</summary>
        Public Property ShadowAlpha As Integer 'Byte型で足りるのだが、Integer型より実行ファイルが大きくなっちゃう。どちらが良いのか分からないので、今回は《Color.FromArgb(～)》に従う。

    End Structure



    '============================================================================================



    ''' <summary>マウスカーソルがトレイアイコンに乗った位置</summary>
    Dim MouseHoverPoint As Point = Point.Empty

    ''' <summary>ホバー継続時間（マウスがアイコンに乗っている時間）</summary>
    Dim WithEvents HoveringTime As New IntegerWithEvent(0)

    ''' <summary>ホバー継続時間ゲージ情報</summary>
    Dim HoverGaugeData As New GaugeParameter With {.Style = GaugeParameter.GaugeStyleType.Horizontal, .BaseRect = New Rectangle(1, 1, 14, 3), .BaseColor = Nothing, .ShadowAlpha = &H80} '色が《Nothing》ならアイコンの絵柄を使う。


    ''' <summary>自動撤収</summary>
    Dim WithEvents AutoCloser As New BooleanWithEvent(False)

    ''' <summary>自動撤収時間カウンタ</summary>
    Dim WithEvents CloseTime As New IntegerWithEvent(0)



    '============================================================================================




    Private Sub ntfTrayIcon_MouseDown(sender As Object, e As System.Windows.Forms.MouseEventArgs) Handles ntfTrayIcon.MouseDown

        '=== ランチメニューの撤収 ===
        If ctxLaunchMenu.Visible Then ctxLaunchMenu.Close()
        '本来なら、タスクトレイアイコンをクリックすると自動的に閉じられるが、ホバー展開後不全症を発症していると閉じてくれないので、強制的に撤収し自動撤収フラグを降ろさせる。

    End Sub

    Private Sub ntfTrayIcon_MouseUp(sender As Object, e As System.Windows.Forms.MouseEventArgs) Handles ntfTrayIcon.MouseUp

        If Not LaunchMenuBuilding.Value Then 'ランチメニュー構築中なら何もしない。

            '=== 手動操作 ===
            If e.Button = Windows.Forms.MouseButtons.Left Then PopupLaunchMenu(True) 'ランチメニュー展開（右クリックは、自動なので不要）

            '--- ホバー監視を停止 ---
            StopHoveringChecker()

        End If

    End Sub

    Private Sub ntfTrayIcon_MouseMove(sender As Object, e As System.Windows.Forms.MouseEventArgs) Handles ntfTrayIcon.MouseMove

        '=== ホバー監視開始不可 其の一 ===
        '条件 ─ 以前の《MouseHoverPoint》と現在のマウスカーソル位置が同じ時
        '         └ メニューをキーボード操作で閉じた直後（アイコンにマウスカーソルが乗ったままの状態）に、ランチメニューが開いてしまうのを防ぐ。そのためには、メニューが開いている等の時でも、《MouseHoverPoint》を更新しなければならない。
        If MouseHoverPoint = System.Windows.Forms.Cursor.Position Then Exit Sub '※※※

        '=== マウスカーソルのスクリーン座標を取得 ===
        MouseHoverPoint = System.Windows.Forms.Cursor.Position

        '=== ホバー監視開始不可 其の二 ===
        '条件 ┬ ランチメニュー構築中
        '     └ ランチメニューまたはシステムメニューを表示中
        If LaunchMenuBuilding.Value OrElse ctxLaunchMenu.Visible OrElse ctxSystemMenu.Visible Then Exit Sub '※※※

        '=== ホバー監視始動（監視中なら継続） ===
        If SystemSettings.TaskTray.HoverPopupper.Enabled AndAlso tmrHoverCloser.Enabled = False Then '《ホバー展開時間 = 0》 → ホバー展開が無効。 / 監視中なら継続するので何もしない。
            'Beep()
            tmrHoverCloser.Start()
        End If

    End Sub

    Private Sub tmrHoverCloser_Tick(sender As System.Object, e As System.EventArgs) Handles tmrHoverCloser.Tick

        If Not AutoCloser.Value Then

            '=== マウスカーソルのホバー監視 ===
            HoverTimer()

        ElseIf Not LaunchMenuBuilding.Value Then 'ランチメニュー構築中はカウントしない

            '=== 自動撤収までの時間をカウント===
            CloserTimer()

        End If

    End Sub

    ''' <summary>
    ''' マウスカーソルのホバー監視
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub HoverTimer()

        '《MouseMove》イベント発生時の座標と同じなら乗っていると判断する。
        If MouseHoverPoint = System.Windows.Forms.Cursor.Position Then

            '--- ホバー展開時間をカウント ---
            If Control.MouseButtons = Windows.Forms.MouseButtons.None Then 'マウスボタンの押下中は、カウントアップしないことで、ボタン開放とランチメニューの表示の衝突を防止する。

                HoveringTime.Value += SystemSettings.TaskTray.HoverPopupper.AdjustStep

                'ホバー継続時間が規定値に達したら、メニューを開く
                If HoveringTime.Value >= SystemSettings.TaskTray.HoverPopupper.Time Then
                    StopHoveringChecker()
                    PopupLaunchMenu(False)

                    '自動撤収機能始動
                    If SystemSettings.TaskTray.AutoCloser.Enabled Then
                        CloseTime.Value = 0
                        AutoCloser.Value = True
                    End If
                End If

            ElseIf HoveringTime.Value > 0 Then '減算することに意味はなく、ただカウントを停止するだけで用は足りる。強いて理由を挙げるなら、見た目・・・。

                HoveringTime.Value -= SystemSettings.TaskTray.HoverPopupper.AdjustStep

            End If

        Else

            '--- タスクトレイアイコン上からマウスが外れていたらタイマー停止 ---
            StopHoveringChecker()
            'Beep()

        End If

    End Sub

    Private Sub HoveringTime_Changed(sender As Object, e As System.EventArgs) Handles HoveringTime.Changed

        '=== ホバー継続時間に変化 ===
        ChangeTaskTrayIcon()

    End Sub

    ''' <summary>
    ''' 自動撤収までの時間をカウント
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub CloserTimer()

        CloseTime.Value += SystemSettings.TaskTray.AutoCloser.AdjustStep

        If CloseTime.Value >= SystemSettings.TaskTray.AutoCloser.Time Then

            '=== ランチメニューを閉じる ===
            AutoCloser.Value = False
            If ctxLaunchMenu.Visible Then ctxLaunchMenu.Close()

        End If

    End Sub

    Private Sub ctxLaunchMenu_Opened(sender As Object, e As System.EventArgs) Handles ctxLaunchMenu.Opened

        '=== 後始末 ===
        LaunchMenuBuilding.Value = False '構築が終了してメニューが表示し終わるまでに、ホバー展開が動き出してしまわないために《Opened》イベントで、フラグを降ろす。

    End Sub

    Private Sub ctxLaunchMenu_Closed(sender As Object, e As System.Windows.Forms.ToolStripDropDownClosedEventArgs) Handles ctxLaunchMenu.Closed

        '=== ランチメニューを閉じた時の後片付け ===
        '--- 自動撤収停止 ---
        AutoCloser.Value = False

        '--- タスクトレイアイコンのコンテキストメニューをシステムメニューに戻しておく ---
        ntfTrayIcon.ContextMenuStrip = ctxSystemMenu

    End Sub

    Private Sub ctxLaunchMenu_PreviewKeyDown(sender As System.Object, e As System.Windows.Forms.PreviewKeyDownEventArgs) Handles ctxLaunchMenu.PreviewKeyDown

        '=== ホバー展開後キー操作されたら自動撤収停止 ===
        '（《KeyDown》イベントでは駄目なんやね。）
        AutoCloser.Value = False

    End Sub

    Private Sub MenuDirectoryItem_DropDownOpening(sender As System.Object, e As System.EventArgs)

        If Not LaunchMenuBuilding.Value Then
            With DirectCast(sender, ToolStripMenuItem)

                '=== 未到格納アイテムを開こうとしたら下層を構築 ===
                If .DropDownItems(0).Tag = UNREACHED_MARK Then 'DropDownItemsの先頭項目を確認。
                    LaunchMenuBuilding.Value = True
                    .Image = My.Resources.rebuild.ToBitmap '構築中マーク
                    My.Application.DoEvents() '《.Image》の変更を確実に反映させる。

                    LaunchMenuLayerBuild(DirectCast(sender, ToolStripMenuItem), 0)
                End If

                '=== 操作アイテムの有無効切り替え ===
                'なぜか、メニューアイテムの《Visible》値が常に《False》で、IF文で違う時だけ設定しなおすという方法が使えない。
                .DropDownItems(OPERATION_ITEM_NAME).Visible = (SystemSettings.LaunchMenu.SubDirectoryOpener OrElse SystemSettings.LaunchMenu.BatchRun)
                .DropDownItems(DIRECTORY_OPEN_NAME).Visible = SystemSettings.LaunchMenu.SubDirectoryOpener
                .DropDownItems(BATCH_RUN_NAME).Visible = SystemSettings.LaunchMenu.BatchRun

            End With
        End If

    End Sub

    Private Sub MenuDirectoryItem_DropDownOpened(sender As System.Object, e As System.EventArgs)

        '=== 未到格納アイテム構築完了 ===
        DirectCast(sender, ToolStripMenuItem).Image = DirectoryItemDummyIcon '構築中マークを消す。ダミー画像を設定しておかないと、大きいアイコン時に、実行アイテムがないメニューを表示すると表示位置が動いてしまう。
        LaunchMenuBuilding.Value = False

    End Sub

    Private Sub ctxLaunchMenu_MouseEnter(sender As Object, e As System.EventArgs) Handles ctxLaunchMenu.MouseEnter

        '=== 自動撤収機能停止 ===
        If AutoCloser.Value Then
            tmrHoverCloser.Stop()
            CloseTime.Value = 0
        End If

    End Sub

    Private Sub ctxLaunchMenu_MouseLeave(sender As Object, e As System.EventArgs) Handles ctxLaunchMenu.MouseLeave

        '=== 自動撤収機能再開 ===
        If AutoCloser.Value Then
            tmrHoverCloser.Start()
        End If

    End Sub

    Private Sub AutoCloser_Changed(sender As Object, e As System.EventArgs) Handles AutoCloser.Changed

        '=== 自動撤収の始動と停止 ===
        With tmrHoverCloser

            If DirectCast(sender, BooleanWithEvent).Value Then
                CloseTime.Value = 0
                .Interval = SystemSettings.TaskTray.AutoCloser.AdjustStep
                .Start()
            Else
                .Stop()
                .Interval = SystemSettings.TaskTray.HoverPopupper.AdjustStep
            End If

        End With

        ChangeTaskTrayIcon()

    End Sub

    Private Sub CloseTime_Changed(sender As Object, e As System.EventArgs) Handles CloseTime.Changed

        '=== 自動撤収へのカウント変化 ===
        ChangeTaskTrayIcon()

    End Sub

    ''' <summary>
    ''' タスクトレイアイコンへのホバー監視を停止する
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub StopHoveringChecker()

        tmrHoverCloser.Stop()
        HoveringTime.Value = 0

    End Sub

    ''' <summary>
    ''' ランチメニューをポップアップする
    ''' </summary>
    ''' <param name="ClickOpenMode">展開モード（クリック展開=[True]/ホバー展開=[False]）</param>
    ''' <remarks></remarks>
    Private Sub PopupLaunchMenu(ClickOpenMode As Boolean)

        '=== ランチメニューを開く時の準備 ===
        'コンテキストメニューの《Opening》イベントで処理すると展開に間に合わず（表示位置が確定してしまっている）、表示位置がずれてしまうみたい。
        'だから、メニュー項目の変更は、表示前に済ませておかないといけない。

        '--- 再構築 ---
        If Not System.IO.Directory.Exists(SystemSettings.LaunchMenu.RegistryPath) Then LaunchMenuAllNew.Value = True 'ランチメニュー登録フォルダが、起動時には有ったのに、いざ開こうとすると無い時の緊急回避策

        If LaunchMenuAllNew.Value Then '必要なら更新
            LaunchMenuBuilding.Value = True
            LaunchMenuRootBuild()
        End If

        '--- 操作アイテムの有無効切り替え ---
        With ctxLaunchMenu
            With .Items(HAND_REBUILD_NAME)
                If Not .Tag Then .Visible = ClickOpenMode 'クリック展開時のみ表示。
            End With
            'ランチメニュー登録フォルダを開く機能は、ルートに最低限必要なので設定にかかわらず常設。
            .Items(BATCH_RUN_NAME).Visible = SystemSettings.LaunchMenu.BatchRun 'よって、一括実行機能のみ切り替える。
        End With

        '--- コンテキストメニューに設定 ---
        ntfTrayIcon.ContextMenuStrip = ctxLaunchMenu

        '=== 開く ===
        '参照 → http://gushwell.ldblog.jp/archives/51968495.html
        '要するに“コントロールのメソッドを抽出して直接呼び出す”ってことみたい。でも、"ShowContextMenu"の部分は、どこから探し出したら良いのかなぁ？
        Dim mInfo As System.Reflection.MethodInfo = ntfTrayIcon.GetType.GetMethod("ShowContextMenu", Reflection.BindingFlags.Instance Or Reflection.BindingFlags.NonPublic)
        mInfo.Invoke(ntfTrayIcon, New Object() {})
        'ctxLaunchMenu.Show(MouseHoverPoint) '表示自体は、《NotifyIcon.ContextMenuStrip.Show(～)》でも可能。

    End Sub

End Class
