﻿'┌────┐
'│技術要綱├──────────────────────
'└┬───┘
'  │『タスクトレイアイコンに一定時間マウスカーソルが乗ったのを知る基本的な考え方』
'  │・残念ながら、《NotifyIcon》クラス(タスクトレイアイコン)には、マウスが重なったり外れたりを知るのに便利な《MouseEnter》や《MouseLeave》イベントがありません。
'  │・マウスカーソルがアイコン内に入ったことは、《MouseMove》イベントで把握できる。そこで、イベント発生時のマウスカーソルの位置をスクリーン座標で記憶し、ホバー時間カウント用タイマーが動いていなかったら動かす。
'  │・ホバー時間カウント用タイマーの割り込み（《Tick》イベント）で、マウスカーソルの位置（スクリーン座標）を取得し、先に記憶しておいた座標と比較する。同じならアイコンに乗ったまま、違うなら外れたと判断できる（アイコン内で移動していても、《MouseMove》イベントで座標が更新されるので、タイミングが悪くない限り対応可能で許容範囲内としちゃう）。
'  │
'  │『タスクトレイアイコン(NotifyIcon)の左右クリックで異なるコンテクストメニューを表示する方法（電源オプションのアイコンのような感じ）・・・“内部メソッド呼び出し方式”』（本作での実装と少々異なる部分有り）
'  │・NotifyIconの《MouseDown》イベントで、押されたボタンに対応するメニューを《ContextMenuStrip》プロパティに設定する。
'  │・NotifyIconの《MouseUp》イベントで、右以外のクリックを受け取ったら、NotifyIconの内部メソッドを直接呼び出して右クリック時と同じコンテキストメニュー展開処理を行わせる（《PopupLaunchMenu()》のコード参照）。もちろん、右クリック時は、自動的に処理される。
'  │・ちなみに、NotifyIconの《MouseDown》/《MouseUp》イベントは、キーボードによる操作でも発生する。（[Enter]=ダブルクリック/[Space]=シングルクリック/[Shift]+[F10] or [アプリケーション]キー=右クリック）
'  │・本作での《ntfTrayIcon.ContextMenuStrip》は、左クリック時のみランチメニューで、それ以外では常にシステムメニューになっている。
'  │※既知の問題
'  │・設定ダイアログのポップアップヘルプを表示中に、ランチメニューをホバーで表示すると、タスクバーにフォームが現れてしまうことがある。
'  │・タスクトレイアイコンをクリックせずにコンテキストメニューを開くと、そのメニュー以外をクリックしても閉じられない場合がある。こうなると、キー操作も受け付けない。Webからの情報により、《.Show()》メソッドが駄目なのかと考え、NotifyIconの内部メソッドを直接呼び出す方法に切り替えたが完治せず。また、《GotFocus》イベントが発生するので、フォーカスを得られていないわけでもないみたい。項目を《Select》しても駄目だった。
'  │  ├→ この症状を、『ホバー展開後不全症』と呼称する。
'  │  ├→ 発症条件は不明。とりあえず、スクリーン（画面）のコンテキストメニューを表示している時にホバー展開すると発症するみたい。
'  │  └→ 対応策として、『自動撤収』を設けた。
'  │※“《ContextMenuStrip.Show(～)》メソッド方式”との違い
'  │・コンテキストメニューの表示自体は、《ContextMenuStrip.Show(～)》メソッドでも可能。
'  │・ただし、“《ContextMenuStrip.Show(～)》メソッド方式”の利点と欠点を勘案して、“内部メソッド呼び出し方式”を採用している。
'  │  ├→ 利点 ─ 《.Show()》メソッドで開いたコンテキストメニューは、タスクバーを避けて表示される。よって、自動撤収時にアイコンのゲージとランチメニューの重なりを考慮する必要がなくなり、設定ダイアログやシステム設定ファイルの項目を減らせる。
'  │  └→ 欠点 ┬ 《.Show()》メソッドでは、表示したコンテキストメニューがアイコンに重ならないので、右クリックで表示した状態と差異が生まれる。
'  │            └ 《.Show()》メソッドで開いたコンテキストメニューを表示した状態でタスクトレイアイコンを右クリックした場合、表示中のメニューが消えるだけで右クリック用コンテキストメニューが表示されない時がある。（“内部メソッド呼び出し方式”で上手く行っているのは、たまたまかもしれないけど・・・。）
'  │
'  │『自動撤収機能の仕組み』
'  │・そもそもタスクトレイアイコンは、アプリケーションとは別の独自のフォームで管理されているのだと考えられる。ためしに、アイコンを右クリックした後[Alt]+[F4]を押すと、アプリケーション本体は終了せず（タスク マネージャで確認）タスクトレイアイコンだけが消えることからも窺える。
'  │・そのため、タスクトレイアイコンをクリックせずにメニューを開くと、隠れフォームにフォーカスが上手く移動せず、ホバー展開後不全症を発症することになる。
'  │・ホバー展開後不全症をどうしても完治させられなくて、困り果てた末に繰り出した最終奥義が、「自動撤収」機能です。
'  │○「自動撤収」は、コンテキストメニュー上からマウスカーソルが出たらカウントを始め、一定時間が経過すると強制的に閉じる。カウント中に再びマウスカーソルが入ると、カウントを停止する。
'  │・マウスカーソルの出入りは、イベントで知るとこができる。ただし、各メニューアイテムのイベントではなく、メニューが置かれるベース部分のイベントを利用する。
'  │  ├ 《ContextMenuStrip》  ・・・ 《ctxLaunchMenu.MouseEnter》              / 《ctxLaunchMenu.MouseLeave》
'  │  └ 《ToolStripMenuItem》 ・・・ 《ToolStripMenuItem.DropDown.MouseEnter》 / 《ToolStripMenuItem.DropDown.MouseLeave》（こちらは、デザイナやコードエディタから直接扱えないので、コード内で動的に関連付ける必要がある。）
'  │
'  │『[裏メニュー]項目の移動方法』
'  │・メニューのアイテムは、どこかにあればいつでも名前で使える。
'  │・ランチメニューは、必要に応じて再構築されるため、ランチメニューへ移動する時は、順番を気にせず持って行けば良い。ランチメニューの構築時は、いったん[裏メニュー]アイテムを変数に退避しておき、必要な項目を順次追加していく。
'  │・システムメニューへは、末尾に置くので素直に追加でOK。
'  │
'  │『コンテキストメニューの逐次構築』
'  │・コンテキストメニューに項目が一つもない状態で表示しようとすると、《Opening》イベントは発生するが、そこでいくら項目を追加しようとも実際には表示してくれない。これを回避するためにダミーの項目を追加しておく。
'  │・更に下層を持つ項目の場合は、《DropDownMenuItems》に未構築を示すアイテムを追加して、《Opening》イベントの関連付けをしておく。ドロップダウンメニューを開こうとしたら《Opening》イベントが発生するので、未構築アイテムの有無をチェックして必要なら構築処理を行う。
'  │○フォルダ構造を逐次構築するメリット
'  │  ├ 1. フォルダ構造を読み込むには、「再帰呼び出し」処理が簡単。しかし、再帰回数が深くなりすぎるとエラー（StackOverflowException）になってしまう。そこで、コンテキストメニューの開かれようとしている層を逐次構築することで、このエラーを回避できる。
'  │  ├ 2. 更に、HDDへのアクセス（メニュー構築に掛かる時間）を分散できる。
'  │  └ 3. あるメニューアイテム以下を再構築することも可能となる。
'  │※既知の問題：ドロップダウンメニューの構築中に他のイベント（項目を選択したり、他のドロップダウンメニューを開いたり）が発生した場合、メニューが表示される前に（《Opening》と《Opened》イベントの間）そのイベントが実行され、その後どろぷダウンメニューが表示されることになる。こうなると、親のメニューは消されドロップダウンメニューのみが取り残されたり、複数のドロップダウンメニューが表示されたりしてしまう。
'  │
'  │『ランチメニューの更新』
'  │・《FileSystemWatcher》クラスで、ランチメニュー登録フォルダを監視する。
'  │・変更アイテムがフォルダであれファイルであれ、その親フォルダに相当する格納アイテム以降を全て再構築する。
'  │  └→ 《FileSystemWatcher》クラスのイベントで変化の内容（追加・削除・改名など）を知れるため、変更箇所のみを個別に対処しようと思えばできるかもしれないけど、そこに手を出すとすごく面倒なことになりそうなので断念。それでも、以前はルートから全再構築だったことに比べると、幾分ましになったと思う。
'  │
'  │『フォーム内で使用して行く仕切り線』
'  │・コントロールを仕切る線として、描画するのではなく、《Label》コントロールに枠（《Fixed3D》）を付けて2px幅にして代用している。理由は、簡単だから。
'  │
'  │『「バージョン情報を閉じる」/「本作を終了する」/「OSが終了（シャットダウンまたはログオフ）する」を区別する方法』
'  │・《FormMain》がバージョン情報を兼ねているので、バージョン情報を表示している時に[×]ボタンや[Alt]+[F4]を押されたら困る。
'  │・意図しない終了を回避するには、《Me.FormClosing》イベントでキャンセル《e.Cancel = True》する方法があるが、これをすると、今度はOSの終了が阻害されてしまう。（OSが終了しようとする時、動作中の全プログラムに「終了せよ！」との命令を送り、各プログラムの終了プロセスが呼び出される。その時、何れかのプログラムに終了を拒まれると、OSも終了できなくなってしまう。）
'  │・OSの終了を察知する方法には、《Microsoft.Win32.SystemEvents.SessionEnding》イベントがある。しかし、《Me.FormClosing》と《Microsoft.Win32.SystemEvents.SessionEnding》は全く別の仕組みであるため、どっちが先に発生するか分からない。よって、この方法では対処不可能である。（要するに、「通常終了」と「ログオフ」を区別したい時意外、《Microsoft.Win32.SystemEvents.SessionEnding》の出番はなさそう・・・。）
'  │○以上のことから下記の点を《Me.FormClosing》で判断して対処する。（そもそも、《Me.FormClosing》で終了理由を知れることを知らなかった。）
'  │⇒「バージョン情報を閉じる」 → 《ntfTrayIcon》が表示されている。
'  │⇒「本作を終了する」         → 《ntfTrayIcon》が表示されていない。（システムメニューの[終了]をクリックした時に消しておく。起動時エラーによる終了の時点では、まだ表示されていない。）
'  │⇒「OSが終了する」           → 《Me.FormClosing》イベントの《e.CloseReason》プロパティの内容で判断する。（MDIやOwnerは存在しないので、「ユーザの操作《CloseReason.UserClosing》」以外をOSによる終了とみなすことにした。ちなみに、「シャットダウン」「再起動」「ログオフ」共に《CloseReason.WindowsShutDown》になる。）
'  └────────────
'┌────┐
'│デザイン├──────────────────────
'└┬───┘
'  │『[裏メニュー]に[開く]と[閉じる]をわざわざ設けている理由』
'  │・現在の鍵の開閉状況と、行おうとする作業を、はっきり区別し認識してもらうため。現況をアイコンで、作業をメニュー項目で表現する。
'  │・現状を表すアイコンの意味を知らせるための見本として、子項目を使う。
'  └────────────





''' <summary>
''' メイン兼バージョン情報
''' </summary>
''' <remarks></remarks>
Public Class FormMain

    ''' <summary>スタートアップに登録するショートカットのパス</summary>
    Private ReadOnly StartupShortCutPath As String = System.IO.Path.Combine(System.Environment.GetFolderPath(Environment.SpecialFolder.Startup),
                                                                            System.IO.Path.GetFileNameWithoutExtension(Application.ExecutablePath) & ".lnk")



    '============================================================================================================================



    ''' <summary>ランチメニュー構築中フラグ</summary>
    Dim WithEvents LaunchMenuBuilding As New BooleanWithEvent(False)

    ''' <summary>ランチメニュー一新フラグ</summary>
    Dim WithEvents LaunchMenuAllNew As New BooleanWithEvent(True) '起動時に必ずメニューを読み込まなければいけないので、初期値は《True》。

    ''' <summary>裏メニュー施錠フラグ</summary>
    Dim WithEvents UnLocked As New BooleanWithEvent(False)



    '============================================================================================



    Private Sub FormMain_Load(sender As System.Object, e As System.EventArgs) Handles MyBase.Load

        '=== 準備 ===
        '--- タスクトレイアイコン ---
        '表示は、フォームの《Show》イベントで行う。
        Dim mc As System.Text.RegularExpressions.Match = System.Text.RegularExpressions.Regex.Match(My.Application.Info.Title, "「.*」") 'アプリケーション名から「」で囲まれた文字列を抜き出す正規表現。
        ntfTrayIcon.Text = mc.Value.Substring(1, mc.Value.Length - 2) '"「"と"」"を外す。

        '--- フォーム（バージョン情報） ---
        Text = ntfTrayIcon.Text & Text

        With My.Application.Info
            Dim title() As String = .Title.Insert(.Title.IndexOf("「"c), vbCrLf).Split(vbCrLf) '文字列で《Split》を行った場合、最初の1文字目で区切られるみたい。例えば、《"0ab1".Split("ab")》とした場合、{"0", "b1"}に分割される。{"0", "1"}を期待しているのだが・・・。
            lblTitle.Text = "=== " & title(0).Trim & " ===" & vbCr & title(1) '上記の《Split》で、Lf（ラインフィード）は残っているので、Cr（キャリッジリターン）のみ追加している。Lfだけでも期待する結果は得られるが、念のため。
            lblVersion.Text = "Version " & .Version.ToString
            lblCopyright.Text = .Copyright
            lnkCompany.Text = .CompanyName
        End With

        With lnkCompany
            .Links.Add(0, .Text.Length, "http://www.h3.dion.ne.jp/~u-tyann0/digitalian_works/")
            ToolTip1.SetToolTip(lnkCompany, .Links(0).LinkData)
        End With

        '--- システム設定パラメータの読み込み ---
        Try
            SettingsLoad(SystemSettingsFileName, SystemSettings)
        Catch ex As Exception
            MessageBox.Show("設定の読み込み・復元中に例外が発生しました。" & vbCrLf & "規定値を使って起動しますので、設定をやり直してください。" & vbCrLf & vbCrLf & ex.Message,
                            ntfTrayIcon.Text,
                            MessageBoxButtons.OK,
                            MessageBoxIcon.Error) '読み込み中にエラーが発生した場合は、読み込めた部分も含め何も反映されないようなので、再初期化は不要。
        End Try

        '--- 鍵の読み込み ---
        Try
            SettingsLoad(keySettingsFileName, KeySettings)
        Catch ex As Exception
            MessageBox.Show("鍵の読み込み・復元中に例外が発生しました。" & vbCrLf & "解除された状態で起動しますので、早急に設定し直してください。" & vbCrLf & vbCrLf & ex.Message,
                            ntfTrayIcon.Text,
                            MessageBoxButtons.OK,
                            MessageBoxIcon.Error) '読み込み中にエラーが発生した場合は、読み込めた部分も含め何も反映されないようなので、再初期化は不要。
        End Try

        '--- 整合性エラーの警告 ---
        With SystemSettings
            Dim errorPoint As String = IIf(.TaskTray.HoverPopupper.AdjustmentErrorCheck, "+・<ホバー展開>/", "") &
                                       IIf(.TaskTray.AutoCloser.AdjustmentErrorCheck, "/+・<自動撤収>/", "") &
                                       IIf(.LaunchMenu.MenuBuild.AdjustmentErrorCheck, "/+・<構築>/", "") &
                                       IIf(.SecretMenu.MouseWheel.AdjustmentErrorCheck, "/+・<マウスホイール>", "")
            'errorPoint = "+*TEST*//+*TEST*"

            If errorPoint <> "" Then
                errorPoint = errorPoint.Replace("//", vbCrLf).Replace("/", "").Replace("+", vbTab)
                MessageBox.Show("システム設定ファイルの記述に致命的な不整合があるため、起動を中止します。" & vbCrLf & "下記の項目に関する設定値を、書き直してください。" & vbCrLf & vbCrLf & errorPoint,
                                ntfTrayIcon.Text,
                                MessageBoxButtons.OK,
                                MessageBoxIcon.Error)

                Me.Close() '起動断念※※※※
                Exit Sub
            End If
        End With

        '--- ランチメニュー ---
        LaunchMenuIconSizeChange()

        '--- [裏メニュー]項目 ---
        If SystemSettings.SecretMenu.Locate = SystemParameter.SecretMenuParameter.LocateNo.LaunchMenu Then SecretMenuArrange()
        LockStatusChange()

        AddHandler tsmSecretMenu.DropDown.MouseEnter, AddressOf ctxLaunchMenu_MouseEnter '┬ 裏メニューがランチメニューに配置されている時に、自動撤収の一時停止を行うため。
        AddHandler tsmSecretMenu.DropDown.MouseLeave, AddressOf ctxLaunchMenu_MouseLeave '┘ （《ctxLaunchMenu_Mouse～》に、《tsmSecretMenu.DropDown.Mouse～》イベントを直接書けない。）

        '--- ホバー ---
        tmrHoverCloser.Interval = SystemSettings.TaskTray.HoverPopupper.AdjustStep

        '--- ランチメニュー登録フォルダ ---
        '存在チェック
        LaunchMenuExistenceCheck(SystemSettings.LaunchMenu.RegistryPath) 'ランチメニューを起動時に読み込まない設定の時でも、起動と同時に登録フォルダが作成されるようにここで実行しておく。

        '監視
        With fswRegistryItemWatcher '《FileSystemWatcher》のプロパティ設定関連には、どうもバグがあるようだ。↓
            .EnableRaisingEvents = False '監視が実行されていると、他のプロパティが変更できない。なのに、デザイン画面のプロパティウィンドウで《False》に設定しても、なぜか次回読み込み時には勝手に《Ture》に書き換えられる。どうやらプロパティウィンドウの見た目だけ《Ture》で内部では《False》になっているようなのだが、何かの拍子に《Ture》化してしまうみたい。原因不明。仕方がないのでここで無効化する。
            .NotifyFilter = IO.NotifyFilters.FileName Or IO.NotifyFilters.DirectoryName Or IO.NotifyFilters.Attributes 'プロパティウィンドウでは複数選択できないので、ここで設定する。
            .IncludeSubdirectories = True
        End With

        '--- 自動起動 ---
#If DEBUG Then
        tsmAutoBoot.ToolTipText &= vbCrLf & """" & System.IO.Path.GetFileName(StartupShortCutPath) & """" 'デバッグ時のみ、ショートカット名を表示
#End If

    End Sub

    Private Sub FormMain_Shown(sender As Object, e As System.EventArgs) Handles Me.Shown

        '=== フォームを隠す ===
        Me.Visible = False '《Load》イベントで非表示にしても《Show》で《Visible》プロパティが《True》になるから、ここでやるしかない。

        '=== ランチメニューの構築 ===
        If SystemSettings.LaunchMenu.MenuBuild.StartUpBuild Then LaunchMenuRootBuild()

        '=== タスクトレイアイコン表示 ===
        ChangeTaskTrayIcon()
        ntfTrayIcon.Visible = True

    End Sub

    Private Sub FormMain_FormClosing(sender As Object, e As System.Windows.Forms.FormClosingEventArgs) Handles Me.FormClosing

        '=== バージョン情報フォームの[×]をクリックした？ ===
        If ntfTrayIcon.Visible AndAlso e.CloseReason = CloseReason.UserClosing Then '《ntfTrayIcon.Visible》が表示されているということは、バージョン情報を閉じるだけの時。また、《e.CloseReason》でOSが終了しようとしている場合を除外する。
            e.Cancel = True '『終了』をキャンセル
            btnClose.PerformClick()
        End If

    End Sub

    Private Sub FormMain_FormClosed(sender As Object, e As System.Windows.Forms.FormClosedEventArgs) Handles Me.FormClosed

        '=== 終了 ===
        fswRegistryItemWatcher.EnableRaisingEvents = False 'これ要るのかなぁ？

    End Sub

    Private Sub tsmExit_Click(sender As System.Object, e As System.EventArgs) Handles tsmExit.Click

        '=== 終了 ===
        ntfTrayIcon.Visible = False '《ntfTrayIcon》の非表示化で、『終了』する意思を表す。
        Me.Close()

    End Sub

    Private Sub tsmVersion_Click(sender As System.Object, e As System.EventArgs) Handles tsmVersion.Click

        '=== バージョン情報 ===
        If Me.Visible Then

            '--- 表示済みならアクティブにする ---
            Me.Activate()

        Else

            '--- 他の項目を選択できないようにする ---
            tsmSetting.Enabled = False '   ┬ バージョン情報は親フォーム《FormMain》であり、子フォームは《ShowDialog》で呼び出される関係上、同時利用できない。
            tsmSecretMenu.Enabled = False '┘

            '--- 表示 ---
            Me.ShowInTaskbar = True
            Me.Opacity = 1
            Me.Visible = True
            '《Visible = False》にすることでタスクバーへ表示されなくなので、《ShowInTaskbar》は初期値《True》のままで良いのだが、《Visible = False》は、《Shown》イベントでしか行えないので、一瞬タスクバーに見えてしまう。
            'そこで、デザイン時に《ShowInTaskbar = False》に設定しておき、バージョン情報を表示する時に《True》にする。後は、フォームの表示状態と連動するので、《ShowInTaskbar = False》に設定しなおす必要はない（けど、念のためやってる）。

        End If

    End Sub

    Private Sub tsmSetting_Click(sender As System.Object, e As System.EventArgs) Handles tsmSetting.Click

        '=== 設定 ===
        Static optionForm As FormOption = Nothing 'フォームを静的変数で用意（フォーム表示中に、再度メニューがクリックされた場合に対処するため）

        If optionForm Is Nothing OrElse optionForm.IsDisposed Then

            '--- 他の項目を選択できないようにする ---
            tsmVersion.Enabled = False 'バージョン情報は親フォーム《FormMain》であり、《ShowDialog》で呼び出す関係上、同時利用できない。
            tsmSecretMenu.Enabled = False '鍵の解除中に鍵が変更されては困るので、同時利用できない。

            '--- ダイアログを表示 ---
            optionForm = New FormOption
            With optionForm

                .Text = ntfTrayIcon.Text & .Text
                .Icon = Me.Icon

                If .ShowDialog(Me) = Windows.Forms.DialogResult.OK Then

                    'システム設定パラメータの保存
                    Try
                        SettingsSave(SystemSettingsFileName, SystemSettings)
                    Catch ex As Exception
                        MessageBox.Show("システム設定の保存中に例外が発生しました。" & vbCrLf & vbCrLf & ex.Message, ntfTrayIcon.Text, MessageBoxButtons.OK, MessageBoxIcon.Error)
                    End Try

                    '変更箇所の反映
                    If (.ChangedFlag And FormOption.ChangedMask.HoverTime) = FormOption.ChangedMask.HoverTime Then 'タスクトレイアイコンの変更が必要か？（ホバー展開時間が、0になった時または0ではなくなった時にゲージの色を変更する必要がある）
                        ChangeTaskTrayIcon()
                    End If

                    If (.ChangedFlag And FormOption.ChangedMask.LargeIcon) = FormOption.ChangedMask.LargeIcon Then 'アイコンサイズが変わったか？
                        LaunchMenuIconSizeChange()
                    End If

                    If (.ChangedFlag And FormOption.ChangedMask.SecretMenuLocate) = FormOption.ChangedMask.SecretMenuLocate Then '[裏メニュー]項目の設置場所が変わったか？
                        SecretMenuArrange()
                    End If

                    If (.ChangedFlag And FormOption.ChangedMask.Key) = FormOption.ChangedMask.Key Then '鍵が新しくなったか？
                        UnLocked.Value = False

                        '鍵の保存（不可視で）
                        Try
                            SettingsSave(keySettingsFileName, KeySettings, True)
                        Catch ex As Exception
                            MessageBox.Show("鍵の保存中に例外が発生しました。" & vbCrLf & vbCrLf & ex.Message, ntfTrayIcon.Text, MessageBoxButtons.OK, MessageBoxIcon.Error)
                        End Try
                    End If

                    If (.ChangedFlag And FormOption.ChangedMask.RegistryDirectoryPath) = FormOption.ChangedMask.RegistryDirectoryPath OrElse
                       (.ChangedFlag And FormOption.ChangedMask.ShowExtension) = FormOption.ChangedMask.ShowExtension OrElse
                       (.ChangedFlag And FormOption.ChangedMask.Key) = FormOption.ChangedMask.Key OrElse
                       (.ChangedFlag And FormOption.ChangedMask.SecretMenuLocate) = FormOption.ChangedMask.SecretMenuLocate Then 'ランチメニューの再構築が必要か？
                        LaunchMenuAllNew.Value = True
                    End If

                End If

            End With

            '--- 後片付け ---
            optionForm.Dispose()
            optionForm = Nothing

            '--- 他の項目を選択できるようにする ---
            tsmVersion.Enabled = True
            tsmSecretMenu.Enabled = True

        Else

            '--- 表示済みならアクティブにする ---
            optionForm.Activate()

        End If

    End Sub

    Private Sub tsmAutoBoot_Click(sender As System.Object, e As System.EventArgs) Handles tsmAutoBoot.Click

        '=== 自動起動 ===
        If DirectCast(sender, ToolStripMenuItem).Checked Then

            '--- 解除 ---
            If MessageBox.Show("Windowsのスタートアップへの登録を解除しますか？", ntfTrayIcon.Text, MessageBoxButtons.YesNo, MessageBoxIcon.Exclamation) = Windows.Forms.DialogResult.Yes Then
                StartupDelete()
            End If

        Else

            '--- 登録 ---
            If MessageBox.Show("Windowsと共に起動するように、スタートアップへ登録しますか？", ntfTrayIcon.Text, MessageBoxButtons.YesNo, MessageBoxIcon.Exclamation) = Windows.Forms.DialogResult.Yes Then
                StartupRegistry()
            End If

        End If

    End Sub

    Private Sub tsmSecretMenuOpen_Click(sender As System.Object, e As System.EventArgs) Handles tsmSecretMenuOpen.Click

        '=== 裏メニューを開く ===
        Static keyForm As FormKey = Nothing 'フォームを静的変数で用意（フォーム表示中に、再度メニューがクリックされた場合に対処するため）

        If keyForm Is Nothing OrElse keyForm.IsDisposed Then

            '--- 他の項目を選択できないようにする ---
            tsmVersion.Enabled = False 'バージョン情報は親フォーム《FormMain》であり、《ShowDialog》で呼び出す関係上、同時利用できない。
            tsmSetting.Enabled = False '鍵の解除中に鍵が変更されては困るので、同時利用できない。

            '--- 鍵認証 ---
            keyForm = New FormKey() With {.Text = ntfTrayIcon.Text & .Text,
                                          .Icon = Me.Icon,
                                          .Location = Windows.Forms.Cursor.Position - New Size(.Width \ 2, .Height \ 2)} 'マウスカーソルが、中心に来るように表示 → 入力しやすい場所 ＝ マウスカーソルの場所 ≒ タスクトレイアイコンの場所（付近） ＝ スクリーンの端
            '[OK]ボタンの中心をマウスカーソルの位置に持って来たい時は、↓の式に換える。
            '.Location = Windows.Forms.Cursor.Position - SystemInformation.BorderSize - New Size(.btnOK.Left + .btnOK.Width \ 2, SystemInformation.CaptionHeight + .btnOK.Top + .btnOK.Height \ 2)

            With keyForm
                If KeySettings.Password & KeySettings.Dial <> "" Then

                    'スクリーンからのはみ出し補正
                    '（《FormKey.ResizeEnd》イベントで処理した方がスッキリするけど、ここでしか必要ないし、ここに記述した方が実行ファイルが小さくなる誘惑に負けた。ちょうどここをいじってる時が88.5KBと89.0KBのせめぎ合いやった。他をいじると簡単に吹っ飛ぶ程度やけど。）
                    Dim limit As Size = New Size(Screen.PrimaryScreen.WorkingArea.Width - .Width, Screen.PrimaryScreen.WorkingArea.Height - .Height) 'フォームを表示できる範囲

                    If .Left < 0 Then '左右限補正
                        .Left = 0
                    ElseIf limit.Width < .Left Then
                        .Left = limit.Width
                    End If

                    If .Top < 0 Then '上下限補正
                        .Top = 0
                    ElseIf limit.Height < .Top Then
                        .Top = limit.Height
                    End If

                    '鍵を入力
                    .ShowDialog(Me)

                Else

                    '未設定
                    .DialogResult = Windows.Forms.DialogResult.OK

                End If
            End With

            '--- 裏メニューの開示を許可 ---
            If keyForm.DialogResult = Windows.Forms.DialogResult.OK Then UnLocked.Value = True

            '--- 後片付け ---
            keyForm.Dispose()
            keyForm = Nothing

            '--- 他の項目を選択できるようにする ---
            tsmVersion.Enabled = True
            tsmSetting.Enabled = True

        Else

            '--- 表示済みならアクティブにする ---
            keyForm.Activate()

        End If

    End Sub

    Private Sub tsmSecretMenuClose_Click(sender As System.Object, e As System.EventArgs) Handles tsmSecretMenuClose.Click

        '=== 裏メニューを閉じる ===
        UnLocked.Value = False

    End Sub

    Private Sub picIcon_Paint(sender As Object, e As System.Windows.Forms.PaintEventArgs) Handles picIcon.Paint

        '=== バージョン情報用アイコン描画 ===
        With e.Graphics
            .DrawIcon(Me.Icon, 0, 0) '32*32Dots
            .DrawIcon(New Icon(Me.Icon, 16, 16), 7, 37) '16*16Dots
        End With

    End Sub

    Private Sub btnClose_Click(sender As System.Object, e As System.EventArgs) Handles btnClose.Click

        '=== バージョン情報を閉じる ===
        Me.Opacity = 0
        Me.Visible = False
        Me.ShowInTaskbar = False '《Visible = False》の後に行わないと、フォームが点滅したようになってしまうみたい。

        '=== 他の項目を選択できるようにする ===
        tsmSetting.Enabled = True
        tsmSecretMenu.Enabled = True

    End Sub

    Private Sub lnkCompany_LinkClicked(sender As System.Object, e As System.Windows.Forms.LinkLabelLinkClickedEventArgs) Handles lnkCompany.LinkClicked

        '=== Web Siteを開く ===
        With CType(sender, LinkLabel)
            .Links(.Links.IndexOf(e.Link)).Visited = True
        End With

        Try
            System.Diagnostics.Process.Start(e.Link.LinkData)
        Catch ex As Exception
            MessageBox.Show("表示に失敗しました。" & vbCrLf & vbCrLf & ex.Message, ntfTrayIcon.Text, MessageBoxButtons.OK, MessageBoxIcon.Error)
        End Try

    End Sub

    Private Sub fswRegistryItemWatcher_Changed(sender As System.Object, e As System.IO.FileSystemEventArgs) Handles fswRegistryItemWatcher.Changed, fswRegistryItemWatcher.Created, fswRegistryItemWatcher.Deleted

        '=== ランチメニューに変更有り ===
        LaunchMenuModify(e.FullPath)

    End Sub

    Private Sub fswRegistryItemWatcher_Renamed(sender As System.Object, e As System.IO.RenamedEventArgs) Handles fswRegistryItemWatcher.Renamed

        '=== ランチメニューに変更有り ===
        LaunchMenuModify(e.FullPath)

    End Sub

    ''' <summary>
    ''' ランチメニューに更新があった
    ''' </summary>
    ''' <param name="Path">更新アイテムのパス</param>
    ''' <remarks></remarks>
    Private Sub LaunchMenuModify(Path As String)

        'ランチメニューが開いていたら閉じる
        If ctxLaunchMenu.Visible Then ctxLaunchMenu.Close()

        '一新フラグが既に立っていたら抜ける
        If LaunchMenuAllNew.Value Then Exit Sub '※※※フラグが立った時点で監視を停止しているのでたいした意味ないけど、何かの時のために。

        '=== 目的のメニューアイテムまでフォルダ名で辿る ===
        Dim errorFlag As Boolean = False 'エラー フラグ
        Dim menuItem As ToolStripMenuItem = Nothing '辿ったメニューアイテム

        For Each element As String In System.IO.Path.GetDirectoryName(Path).Replace(SystemSettings.LaunchMenu.RegistryPath, "").Split({System.IO.Path.DirectorySeparatorChar}, System.StringSplitOptions.RemoveEmptyEntries) '1.変更アイテムの親フォルダを取得して、2.ルート部分を除去して、3.各フォルダ名を分割（空白は無視）。
            '                         1~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 2~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 3~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

            '--- フォルダに対応するメニューアイテムを取得 ---
            If menuItem Is Nothing Then
                menuItem = ctxLaunchMenu.Items(element) '1層目の場合
            Else
                menuItem = menuItem.DropDownItems(element) '2層目以降の場合
            End If

            'メニューアイテムが見つからないのはエラー
            If IsNothing(menuItem) Then
                errorFlag = True
                Exit For '※※
            End If

            '目的のアイテムまたはその親が未構築アイテムだったら改めて対処する必要がない
            If menuItem.DropDownItems(menuItem.DropDownItems.Count - 1).Tag = UNREACHED_MARK Then Exit Sub '※※※

        Next

        '--- 未到アイテム化 ---
        If menuItem Is Nothing Then
            LaunchMenuAllNew.Value = True 'エラーまたはトレース結果が《Nothing》ならルートのアイテムに変化があったことを示す → 一新フラグを立てる。（エラーメッセージを表示する前に再構築フラグを立てておかないと、メッセージを表示したまま旧メニューを操作できてしまう。）
            If errorFlag Then MessageBox.Show("ランチメニューから変更箇所を検索しましたが、見付けられませんでした。" & vbCrLf & "ランチメニュー登録フォルダに設定されているディレクトリ パスに問題がないか確認してください。", ntfTrayIcon.Text, MessageBoxButtons.OK, MessageBoxIcon.Error)
        Else
            SetUnreachedTag(menuItem) '未到情報を書き込む
        End If

    End Sub

    Private Sub ctxSysMenu_Opening(sender As Object, e As System.ComponentModel.CancelEventArgs) Handles ctxSystemMenu.Opening

        If LaunchMenuBuilding.Value Then
            '=== 構築中は開かない ===
            e.Cancel = True
        Else
            '=== スタートアップへの登録状況チェック ===
            StartupChecker()
        End If

    End Sub

    Private Sub LaunchMenuAllNew_Changed(sender As Object, e As System.EventArgs) Handles LaunchMenuAllNew.Changed

        With DirectCast(sender, BooleanWithEvent)

            '=== ランチメニューが開いていたら閉じる ===
            If .Value AndAlso ctxLaunchMenu.Visible Then ctxLaunchMenu.Close()

            '=== ランチメニューの一新フラグに変化 ===
            ChangeTaskTrayIcon()

            '=== ランチメニュー登録フォルダの変更監視の停止 ===
            '（次回再構築するので、変更を監視する意味がない）
            If .Value Then fswRegistryItemWatcher.EnableRaisingEvents = False

        End With

    End Sub

    Private Sub LaunchMenuBuilding_Changed(sender As Object, e As System.EventArgs) Handles LaunchMenuBuilding.Changed

        '=== ランチメニューの更新中フラグに変化 ===
        ChangeTaskTrayIcon()

    End Sub

    Private Sub UnLocked_Changed(sender As Object, e As System.EventArgs) Handles UnLocked.Changed

        '=== 裏メニューの開閉状態が変更された ===
        LockStatusChange()
        LaunchMenuAllNew.Value = True

    End Sub

    ''' <summary>
    ''' タスクトレイ用アイコンを作成して変更する
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub ChangeTaskTrayIcon()

        '=== ゲージ ===
        Dim gaugeValue As Single 'ゲージ値の計算
        Dim maskAlpha As Integer '目隠し色の透明度
        Dim maskRect As Rectangle '目隠し描画形状

        '--- 描画形状の作成 ---
        With HoverGaugeData

            If SystemSettings.TaskTray.HoverPopupper.Enabled Then

                If AutoCloser.Value Then
                    '自動撤収時間
                    gaugeValue = CloseTime.Value / SystemSettings.TaskTray.AutoCloser.Time
                Else
                    'ホバー継続時間
                    gaugeValue = HoveringTime.Value / SystemSettings.TaskTray.HoverPopupper.Time
                End If

                maskAlpha = .ShadowAlpha

                Select Case .Style

                    Case GaugeParameter.GaugeStyleType.Horizontal
                        '横向き
                        With .BaseRect
                            gaugeValue *= .Width
                            maskRect = New Rectangle(.X + .Width - gaugeValue, .Y, gaugeValue, .Height)
                        End With

                    Case Else
                        '縦向き
                        With .BaseRect
                            gaugeValue *= .Height
                            maskRect = New Rectangle(.X, .Y, .Width, gaugeValue)
                        End With

                End Select
                'ホバー展開時間や自動撤収の設定によっては（監視時間で割り切れない場合）、ゲージをはみ出す可能性がある。
                'しかし、直後にメニューが開かれゼロクリアされるので、実質的に影響は少ない。
                'ホバー継続時間や自動撤収時間のはみ出しを矯正していないので、減算時に負数へ行くこともない。

            Else

                'ホバー展開停止中
                maskAlpha = &HFF 'ゲージをブラックアウト
                maskRect = .BaseRect

            End If

        End With

        '=== アイコン作成 ===
        '--- 基本画像の準備 ---
        Dim bmp As Bitmap = My.Resources.ready.ToBitmap

        With Graphics.FromImage(bmp)

            '--- ゲージ描画 ---
            If HoverGaugeData.BaseColor <> Nothing Then .FillRectangle(New SolidBrush(HoverGaugeData.BaseColor), HoverGaugeData.BaseRect) '色指定されているなら棒を描画する。
            .FillRectangle(New SolidBrush(Color.FromArgb(maskAlpha, Color.Black)), maskRect) 'アイコンに予め描かれている棒を目隠し（黒で上塗り）してゲージを表現する。

            '--- 各種マーク合成 ---
            'ランチメニュー再構築中
            If LaunchMenuBuilding.Value Then .DrawIcon(My.Resources.rebuild, 0, 0)

            'ランチメニュー変更有り
            If LaunchMenuAllNew.Value Then .DrawIcon(My.Resources.modify, 0, 0)

        End With

        '=== タスクトレイアイコンに反映 ===
        If AutoCloser.Value Then bmp.RotateFlip(SystemSettings.TaskTray.AutoCloser.IconRotate) '自動撤収の作動時のアイコン回転・反転。
        '●アイコンを回転・反転する理由
        'その1.ホバー展開すると、アイコン上にランチメニューが重なるので、自動撤収までの時間を表すゲージが見えなくなるのを避けたい。（ホバー時は、マウスカーソルと重なるのを避けたいので、ゲージをアイコン上部に表示している。）
        'その2.自動撤収が働いているとこを知らせたい。（ランチメニューを開くのに対して閉じるという逆の動作を表す）

        ntfTrayIcon.Icon = System.Drawing.Icon.FromHandle(bmp.GetHicon) 'BitmapをIconに変換して登録。
        My.Application.DoEvents() 'タスクトレイの更新を行わせる（ホバー継続時間ゲージの満タン状態を見えるようにする）。無くても支障はない。

    End Sub

    ''' <summary>
    ''' [裏メニュー]項目の配置を変更する
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub SecretMenuArrange()

        If SystemSettings.SecretMenu.Locate = SystemParameter.SecretMenuParameter.LocateNo.LaunchMenu Then

            '=== ランチメニューへ ===
            'ランチメニューの構築時に、順序が整理されるので、ここではとりあえず移動するだけで良い。
            ctxLaunchMenu.Items.Add(tssSecretMenu)
            ctxLaunchMenu.Items.Add(tsmSecretMenu)

        ElseIf SystemSettings.SecretMenu.Locate = SystemParameter.SecretMenuParameter.LocateNo.SystemMenu Then

            '=== システムメニューへ ===
            ctxSystemMenu.Items.Add(tssSecretMenu)
            ctxSystemMenu.Items.Add(tsmSecretMenu)

        End If

    End Sub

    ''' <summary>
    ''' 裏メニューの施錠状態をメニューアイコンに反映
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub LockStatusChange()

        tsmSecretMenu.Image = IIf(UnLocked.Value, My.Resources.open, My.Resources.close)

        tsmSecretMenuOpen.Enabled = Not UnLocked.Value
        tsmSecretMenuClose.Enabled = UnLocked.Value

    End Sub

    ''' <summary>
    ''' ランチメニューのアイコンサイズを変更
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub LaunchMenuIconSizeChange()

        Dim s As Byte = IIf(SystemSettings.LaunchMenu.LargeIcon, 32, 16)
        Me.ctxLaunchMenu.ImageScalingSize = New Size(s, s)

    End Sub

    ''' <summary>
    ''' ランチメニュー登録フォルダ存在チェック
    ''' </summary>
    ''' <param name="DirectoryPath">ランチメニュー登録フォルダのパス</param>
    ''' <returns>存在フラグ</returns>
    ''' <remarks></remarks>
    Public Function LaunchMenuExistenceCheck(DirectoryPath As String) As Boolean

        '=== フォルダが存在しなければ作成する ===
        Dim ret As Boolean = True '結局、フォルダは有るのか無いのか

        If Not System.IO.Directory.Exists(DirectoryPath) Then
            Try

                System.IO.Directory.CreateDirectory(DirectoryPath)
                ntfTrayIcon.ShowBalloonTip(5000, "お知らせ", "ランチメニュー登録フォルダを作成しました。", ToolTipIcon.Info)

            Catch ex As Exception

                MessageBox.Show(String.Format("ランチメニュー登録フォルダの作成に、失敗しました。{0}『{1}』{0}{2}", vbCrLf & vbCrLf, ex.Message, DirectoryPath),
                                ntfTrayIcon.Text,
                                MessageBoxButtons.OK,
                                MessageBoxIcon.Error)
                ret = False

            End Try
        End If

        Return ret

    End Function

    ''' <summary>
    ''' スタートアップへの登録状況を確認
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub StartupChecker()

        tsmAutoBoot.Checked = System.IO.File.Exists(StartupShortCutPath)

    End Sub

    ''' <summary>
    ''' スタートアップに登録
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub StartupRegistry()

        If Not System.IO.Directory.Exists(System.IO.Path.GetDirectoryName(StartupShortCutPath)) Then

            MessageBox.Show("スタートアップのフォルダがありません。" & vbCrLf & "設定をやり直してみるか、Windowsの設定を見直してください。" & vbCrLf & vbCrLf & System.IO.Path.GetDirectoryName(StartupShortCutPath), Me.Text, MessageBoxButtons.OK, MessageBoxIcon.Error)
            'ショートカット作成時にどんなエラーが発生するのかわからないので、念のためスタートアップフォルダの存在チェックも入れてみた。
            'スタートアップのパスをシステムメニュー表示時に取得しているので、確定前にスタートアップの場所が変更されると、困ったことのなるのだろうか？

        ElseIf System.IO.File.Exists(StartupShortCutPath) Then

            MessageBox.Show("既にスタートアップに存在しています。", Me.Text, MessageBoxButtons.OK, MessageBoxIcon.Information)

        Else

            '=== ショートカット作成 ===
            Try

                Dim oShell As Object = CreateObject("WScript.Shell")
                Dim oShortcut As Object = oShell.CreateShortcut(StartupShortCutPath) 'ショートカットのファイル名

                With oShortcut
                    .TargetPath = Application.ExecutablePath 'リンク先のファイル名
                    .WorkingDirectory = Application.StartupPath
                    .Description = My.Application.Info.Description 'コメント
                    '.Arguments = "/a /b /c" 'コマンドパラメータ（"リンク先"の後ろに付く）
                    '.Hotkey = "Ctrl+Alt+Shift+F12" 'ショートカットキー（ホットキー）
                    '.WindowStyle = 1 '実行時の大きさ（1=通常, 3=最大化, 7=最小化）
                    '.IconLocation = Application.ExecutablePath + ",0" 'アイコンのパス（自分のEXEファイルのインデックス0のアイコン）
                    '↑色々設定できるけど、今回は不要なのでコメントアウト！！

                    .Save() '作成
                End With

                '参考
                'DOBON.NET > プログラミング道 > .NET Tips > ファイル、フォルダ > ショートカットを作成する ・・・ http://dobon.net/vb/dotnet/file/createshortcut.html
                'Visual Basic 中学校 > VB.NET サンプル > ショートカットを作成する                         ・・・ http://homepage1.nifty.com/rucio/main/dotnet/Samples/SmpleCnt.htm

            Catch ex As Exception

                'Object型の操作なので、この例外処理は意味がない？
                MessageBox.Show("スタートアップへの登録に失敗しました。" & vbCrLf & vbCrLf & ex.Message, Me.Text, MessageBoxButtons.OK, MessageBoxIcon.Error)

            End Try

        End If

    End Sub

    ''' <summary>
    ''' スタートアップから削除
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub StartupDelete()

        If Not System.IO.File.Exists(StartupShortCutPath) Then

            MessageBox.Show("既にスタートアップに存在していません。", Me.Text, MessageBoxButtons.OK, MessageBoxIcon.Information)

        Else

            '=== ショートカット削除 ===
            Try

                My.Computer.FileSystem.DeleteFile(StartupShortCutPath, FileIO.UIOption.OnlyErrorDialogs, FileIO.RecycleOption.DeletePermanently)

            Catch ex As Exception

                MessageBox.Show("スタートアップからの削除に失敗しました。" & vbCrLf & vbCrLf & ex.Message, Me.Text, MessageBoxButtons.OK, MessageBoxIcon.Error)

            End Try

        End If

    End Sub

End Class





'||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||





''' <summary>
''' 変更イベントがあるInteger型
''' </summary>
''' <remarks></remarks>
Public Class IntegerWithEvent

    ''' <summary>
    ''' 値が変わった
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Public Event Changed(sender As System.Object, e As System.EventArgs)



    '============================================================================================



    Private _Value As Integer

    ''' <summary>
    ''' 値
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Property Value As Integer

        Get

            Return _Value

        End Get

        Set(value As Integer)

            If _Value <> value Then
                _Value = value
                RaiseEvent Changed(Me, New EventArgs)
            End If

        End Set

    End Property



    '============================================================================================



    ''' <summary>
    ''' インスタンス
    ''' </summary>
    ''' <param name="Initialize">初期値</param>
    ''' <remarks></remarks>
    Public Sub New(Initialize As Integer)
        _Value = Initialize
    End Sub

End Class





'||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||





''' <summary>
''' 変更イベントがあるBoolean型
''' </summary>
''' <remarks></remarks>
Public Class BooleanWithEvent

    ''' <summary>
    ''' 値が変わった
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Public Event Changed(sender As System.Object, e As System.EventArgs)



    '============================================================================================



    Private _Value As Boolean

    ''' <summary>
    ''' 値
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Property Value As Boolean

        Get

            Return _Value

        End Get

        Set(value As Boolean)

            If _Value <> value Then
                _Value = value
                RaiseEvent Changed(Me, New EventArgs)
            End If

        End Set

    End Property



    '============================================================================================



    ''' <summary>
    ''' インスタンス
    ''' </summary>
    ''' <param name="Initialize">初期値</param>
    ''' <remarks></remarks>
    Public Sub New(Initialize As Boolean)
        _Value = Initialize
    End Sub

End Class