﻿''' <summary>
''' 設定
''' </summary>
''' <remarks></remarks>
Module Setting

    ''' <summary>システム設定ファイル名</summary>
    Public ReadOnly SystemSettingsFileName As String = System.IO.Path.GetFullPath(".\SystemSetting.xml")

    ''' <summary>鍵設定ファイル名</summary>
    Public ReadOnly keySettingsFileName As String = System.IO.Path.GetFullPath(".\KeySetting.xml")

    '起動中に何らかの理由によりカレントディレクトリが変更されてもいいように、起動時にパスを確定する。起動中に実行ファイルが移動（カット&ペーストやドラッグ等）してもカレントディレクトリは変更されないので、この1回で十分。
    'カレントディレクトリを使用することで、ショートカットアイコンの”作業フォルダ”で保存先を指定できるようになる。（ちなみに、《System.IO.Directory.GetCurrentDirectory()》でカレントディレクトリの設定・取得が可能。）
    'ファイル名の前の".\"は、別になくてもいいけどなんとなく付けてみた。ちなみに、"."は現在のフォルダ（即ちカレントフォルダ）、".."なら1つ上のフォルダを意味する。


    ''' <summary>システム設定パラメータ</summary>
    Public SystemSettings As New SystemParameter

    ''' <summary>鍵設定パラメータ</summary>
    Public KeySettings As New KeyParameter


    '::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::


#Region "ファイルアクセス"

    ''' <summary>
    ''' 設定パラメータの復元
    ''' </summary>
    ''' <param name="FileName">ファイル名</param>
    ''' <param name="LoadObject">復元先</param>
    ''' <remarks></remarks>
    Public Sub SettingsLoad(FileName As String, ByRef LoadObject As Object)

        '=== XMLファイルから読み込む ===
        If System.IO.File.Exists(FileName) Then

            Dim fs As System.IO.FileStream = Nothing

            Try

                '--- XMLファイルから読み込み、逆シリアル化する ---
                Dim serializer As New System.Xml.Serialization.XmlSerializer(LoadObject.GetType) 'XmlSerializerオブジェクトの作成
                fs = New System.IO.FileStream(FileName, System.IO.FileMode.Open) 'ファイルを開く

                LoadObject = CTypeDynamic(serializer.Deserialize(fs), LoadObject.GetType)

            Catch ex As Exception

                Throw 'エラーを再スロー

            Finally

                '--- 閉じる ---
                If fs IsNot Nothing Then fs.Close()

            End Try

        End If

    End Sub

    ''' <summary>
    ''' 設定パラメータの保存
    ''' </summary>
    ''' <param name="FileName">ファイル名</param>
    ''' <param name="SaveObject">保存元</param>
    ''' <param name="HideMode">不可視属性モード（《Ture》 = 不可視化 / 《False》 = 現状維持）</param>
    ''' <remarks></remarks>
    Public Sub SettingsSave(FileName As String, SaveObject As Object, Optional HideMode As Boolean = False)

        '=== XMLファイルに書き込む ===
        Dim fs As System.IO.FileStream = Nothing

        Try

            '---不可視属性の解除（しないと書き込めない） ---
            If System.IO.File.Exists(FileName) Then
                Dim fi As New System.IO.FileInfo(FileName)
                If (fi.Attributes And IO.FileAttributes.Hidden) = IO.FileAttributes.Hidden Then
                    fi.Attributes = fi.Attributes And Not IO.FileAttributes.Hidden
                    HideMode = True
                End If
            End If

            '--- シリアル化し、XMLファイルに保存する ---
            Dim serializer As New System.Xml.Serialization.XmlSerializer(SaveObject.GetType) 'XmlSerializerオブジェクトを作成
            fs = New System.IO.FileStream(FileName, System.IO.FileMode.Create) 'ファイルを開く

            serializer.Serialize(fs, SaveObject)

            '---不可視属性の設定・復元 ---
            If HideMode Then
                Dim fi As New System.IO.FileInfo(FileName) '解除時のがあれば再インスタンシング不要かもしれないけど、ファイルが置き換わってるし念のためやっとく。使いまわしても、実質的に問題なさそうやけど。
                fi.Attributes = fi.Attributes Or IO.FileAttributes.Hidden
            End If

        Catch ex As Exception

            Throw 'エラーを再スロー

        Finally

            '--- 閉じる ---
            If fs IsNot Nothing Then fs.Close()

        End Try

    End Sub

#End Region

End Module





'||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||





#Region "設定項目の定義"

''' <summary>
''' システム設定パラメータ
''' </summary>
''' <remarks></remarks>
<System.Xml.Serialization.XmlRoot("LaunchTimeシステム設定パラメータ")> _
Public Class SystemParameter

    ''' <summary>
    ''' タスクトレイに関する設定
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <System.Xml.Serialization.XmlElement("タスクトレイ")> _
    Public Property TaskTray As New TaskTrayParameter

    ''' <summary>
    ''' ランチメニューに関する設定
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <System.Xml.Serialization.XmlElement("ランチメニュー")> _
    Public Property LaunchMenu As New LaunchMenuParameter

    ''' <summary>
    ''' 裏メニューに関する設定
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <System.Xml.Serialization.XmlElement("裏メニュー")> _
    Public Property SecretMenu As New SecretMenuParameter





    '||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||





    ''' <summary>
    ''' タスクトレイに関する設定値
    ''' </summary>
    ''' <remarks></remarks>
    Public Class TaskTrayParameter

        ''' <summary>
        ''' ホバー展開に関する設定
        ''' </summary>
        ''' <value></value>
        ''' <returns></returns>
        ''' <remarks></remarks>
        <System.Xml.Serialization.XmlElement("ホバー展開")> _
        Public Property HoverPopupper As New TimeParameter With {.AdjustMax = 5000, .AdjustStep = 100, .Time = 1000}

        ''' <summary>
        ''' 自動撤収に関する設定
        ''' </summary>
        ''' <value></value>
        ''' <returns></returns>
        ''' <remarks></remarks>
        <System.Xml.Serialization.XmlElement("自動撤収")> _
        Public Property AutoCloser As New AutoCloserParameter With {.AdjustMax = 30000, .AdjustStep = 1000, .Time = 5000}





        '||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||





        ''' <summary>
        ''' 時間に関する設定値（コンストラクタを記述したクラスをシリアル化するとエラーになるから、宣言時に初期値設定するしかない。）
        ''' </summary>
        ''' <remarks></remarks>
        Public Class TimeParameter

            ''' <summary>
            ''' 調整できる上限値
            ''' </summary>
            ''' <value></value>
            ''' <returns></returns>
            ''' <remarks></remarks>
            <System.Xml.Serialization.XmlElement("上限値")> _
            Public Property AdjustMax As Integer = 10000

            ''' <summary>
            ''' 調整できる間隔
            ''' </summary>
            ''' <value></value>
            ''' <returns></returns>
            ''' <remarks></remarks>
            <System.Xml.Serialization.XmlElement("調整間隔")> _
            Public Property AdjustStep As Integer = 500

            ''' <summary>
            ''' 時間
            ''' </summary>
            ''' <value></value>
            ''' <returns></returns>
            ''' <remarks></remarks>
            <System.Xml.Serialization.XmlElement("時間")> _
            Public Property Time As Integer = 2500

            ''' <summary>
            ''' 有効かどうかを知る。
            ''' </summary>
            ''' <value></value>
            ''' <returns></returns>
            ''' <remarks></remarks>
            Public ReadOnly Property Enabled As Boolean

                Get
                    Return (Time > 0)
                End Get

            End Property

            ''' <summary>
            ''' 整合性のチェック
            ''' </summary>
            ''' <returns>致命的な不整合（設定フォームにない項目の場合）</returns>
            ''' <remarks></remarks>
            Public Function AdjustmentErrorCheck() As Boolean

                Dim ret As Boolean = False

                '=== 上限値（禁則処理：0以上） ===
                If AdjustMax < 0 Then ret = True

                '=== 調整間隔（禁則処理：1以上） ===
                If AdjustStep < 1 Then ret = True

                '=== 時間（禁則処理：0以上且つ《上限値》以下） ===
                Select Case Time
                    Case Is < 0
                        Time = 0
                    Case Is > AdjustMax
                        Time = AdjustMax
                End Select

                Return ret

            End Function

        End Class


        '----------------------------------------------------------------------------------------------------------------------------


        ''' <summary>
        ''' 自動撤収に関する設定値
        ''' </summary>
        ''' <remarks></remarks>
        Public Class AutoCloserParameter

            Inherits TimeParameter


            '::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::


            ''' <summary>
            ''' 自動撤収におけるアイコン回転の設定
            ''' </summary>
            ''' <value></value>
            ''' <returns></returns>
            ''' <remarks></remarks>
            <System.Xml.Serialization.XmlElement("アイコン回転")> _
            Public Property IconRotate As System.Drawing.RotateFlipType = RotateFlipType.Rotate180FlipNone
            '推奨設定値 → 基本的にタスクバーの配置場所によって変更する。
            ' ├ 上・・・RotateFlipType.RotateNoneFlipX
            ' ├ 右・・・RotateFlipType.Rotate90FlipNone
            ' ├ 下・・・RotateFlipType.Rotate180FlipNone
            ' └ 左・・・RotateFlipType.Rotate270FlipNone

        End Class

    End Class



    '============================================================================================================================



    ''' <summary>
    ''' ランチメニューに関する設定値
    ''' </summary>
    ''' <remarks></remarks>
    Public Class LaunchMenuParameter

        Private _RegistryPath As String = System.IO.Path.Combine(My.Application.Info.DirectoryPath, "LaunchMenu")


        '::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::


        ''' <summary>
        ''' ランチメニュー登録フォルダの基点パス
        ''' </summary>
        ''' <value></value>
        ''' <returns></returns>
        ''' <remarks></remarks>
        <System.Xml.Serialization.XmlElement("ランチメニュー登録フォルダ")> _
        Public Property RegistryPath As String

            Get
                Return _RegistryPath
            End Get

            Set(value As String)
                _RegistryPath = value

                '=== パスの整形 ===
                '--- パス中に含まれる複数の連続するディレクトリ区切り文字("\\")を1文字に収斂 ---
                '.NETのフォルダ・ファイル操作系は、パスの途中に"\\"があっても、"\"一つとして解釈してくれるみたい。
                'でも、ランチメニューの登録内容変更時に当該箇所を辿る時に少々困る（エラーになる）ので、訂正しておく。（エラーになっても良いけど、.NETさんが許容してるので・・・）
                Dim ngWord As String = System.IO.Path.DirectorySeparatorChar & System.IO.Path.DirectorySeparatorChar
                While _RegistryPath.IndexOf(ngWord) > -1
                    _RegistryPath = _RegistryPath.Replace(ngWord, System.IO.Path.DirectorySeparatorChar)
                End While
                '    ↑・《While ～ End While》で必要十分。《Wend》が《End While》になってるし、《Exit While》も新設されてるし・・・改良されているということは、Microsoft的には現役なの？
                'どっりがお好み？
                '    ↓・《While ～ End While》は古臭いの？ 《Exit Do》の優位性はもうない。複雑な条件式が必要な場面って滅多に遭遇しないなぁ。
                'Do While _RegistryPath.IndexOf(miss) > -1
                '    _RegistryPath = _RegistryPath.Replace(miss, System.IO.Path.DirectorySeparatorChar)
                'Loop

                '--- 末尾のディレクトリ区切り文字("\")を削除 ---
                'ランチメニューの登録内容変更箇所のトレース時に親フォルダを取得するのに邪魔なので取っておく。
                If _RegistryPath.Last = System.IO.Path.DirectorySeparatorChar Then _RegistryPath = Strings.Left(_RegistryPath, _RegistryPath.Length - 1)
            End Set

        End Property

        ''' <summary>
        ''' 構築に関する設定
        ''' </summary>
        ''' <value></value>
        ''' <returns></returns>
        ''' <remarks></remarks>
        <System.Xml.Serialization.XmlElement("構築")> _
        Public Property MenuBuild As New MenuBuildParameter

        ''' <summary>
        ''' 一括実行の許可
        ''' </summary>
        ''' <value></value>
        ''' <returns></returns>
        ''' <remarks></remarks>
        <System.Xml.Serialization.XmlElement("一括実行")> _
        Public Property BatchRun As Boolean = True

        ''' <summary>
        ''' ランチメニュー登録フォルダのサブフォルダを開けるようにする
        ''' </summary>
        ''' <value></value>
        ''' <returns></returns>
        ''' <remarks></remarks>
        <System.Xml.Serialization.XmlElement("登録サブフォルダ展開")> _
        Public Property SubDirectoryOpener As Boolean = True

        ''' <summary>
        ''' 手動撤収機能の常設化
        ''' </summary>
        ''' <value></value>
        ''' <returns></returns>
        ''' <remarks></remarks>
        <System.Xml.Serialization.XmlElement("手動撤収")> _
        Public Property HandClose As Boolean = True

        ''' <summary>
        ''' 格納アイテムの背景色
        ''' </summary>
        ''' <value></value>
        ''' <returns></returns>
        ''' <remarks></remarks>
        <System.Xml.Serialization.XmlElement("格納アイテム色")> _
        Public Property DirectoryItemColor As New SavableColor

        ''' <summary>
        ''' 大きいアイコン表示の有効化
        ''' </summary>
        ''' <value></value>
        ''' <returns></returns>
        ''' <remarks></remarks>
        <System.Xml.Serialization.XmlElement("大アイコン")> _
        Public Property LargeIcon As Boolean = False

        ''' <summary>
        ''' アイテムの登録名に拡張子を表示
        ''' </summary>
        ''' <value></value>
        ''' <returns></returns>
        ''' <remarks></remarks>
        <System.Xml.Serialization.XmlElement("拡張子表示")> _
        Public Property ShowExtension As Boolean = False

        ''' <summary>
        ''' 拡張コマンドライン引数モードの常用化
        ''' </summary>
        ''' <value></value>
        ''' <returns></returns>
        ''' <remarks></remarks>
        <System.Xml.Serialization.XmlElement("拡張コマンドライン引数")> _
        Public Property ExCommandLineOption As Boolean = False





        '||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||





        ''' <summary>
        ''' メニュー構築に関する設定値
        ''' </summary>
        ''' <remarks></remarks>
        Public Class MenuBuildParameter

            ''' <summary>
            ''' 再帰呼び出しの限界深度（[StackOverflowException]回避用、10層も潜れば十分実用に耐える？）
            ''' </summary>
            ''' <value></value>
            ''' <returns></returns>
            ''' <remarks></remarks>
            <System.Xml.Serialization.XmlElement("再帰呼出限界深度")> _
            Public Property RecursiveCallMax As Integer = 10

            ''' <summary>
            ''' 一括で構築する深度
            ''' </summary>
            ''' <value></value>
            ''' <returns></returns>
            ''' <remarks></remarks>
            <System.Xml.Serialization.XmlElement("一括構築深度")> _
            Public Property Depth As Integer = 0

            ''' <summary>
            ''' 起動時にメニューを構築する
            ''' </summary>
            ''' <value></value>
            ''' <returns></returns>
            ''' <remarks></remarks>
            <System.Xml.Serialization.XmlElement("起動時構築")> _
            Public Property StartUpBuild As Boolean = False

            ''' <summary>
            ''' 整合性のチェック
            ''' </summary>
            ''' <returns>致命的な不整合（設定フォームにない項目の場合）</returns>
            ''' <remarks></remarks>
            Public Function AdjustmentErrorCheck() As Boolean

                Dim ret As Boolean = False

                '=== 再帰呼出限界深度（禁則処理：0以上） ===
                If RecursiveCallMax < 0 Then ret = True

                '=== 一括構築深度（禁則処理：0以上且つ《再帰呼出限界深度》以下） ===
                Select Case Depth
                    Case Is < 0
                        Depth = 0
                    Case Is > RecursiveCallMax
                        Depth = RecursiveCallMax
                End Select

                Return ret

            End Function

        End Class


        '----------------------------------------------------------------------------------------------------------------------------


        ''' <summary>
        ''' 保存可能なColor型
        ''' </summary>
        ''' <remarks></remarks>
        Public Class SavableColor

            'Color構造体は、XMLに出力されない（項目名だけ書き出される）ので、文字列型としてカラーコードを読み書きする。

            ''' <summary>
            ''' 色
            ''' </summary>
            ''' <value></value>
            ''' <returns></returns>
            ''' <remarks></remarks>
            <System.Xml.Serialization.XmlIgnoreAttribute()> _
            Public Property Color As Color = Color.FromArgb(&H40, Color.Yellow)

            ''' <summary>
            ''' 色番号（16進数）
            ''' </summary>
            ''' <value></value>
            ''' <returns></returns>
            ''' <remarks></remarks>
            <System.Xml.Serialization.XmlElement("色番号")> _
            Public Property ColorCode As String

                Get
                    Return Convert.ToString(Color.ToArgb, 16) 'Hex(Color.ToArgb)と同じ
                End Get

                Set(value As String)
                    Color = Color.FromArgb(Convert.ToInt32(value, 16)) 'Val("&H" & value)とかCInt("&H" & value)と同じ
                End Set

            End Property

        End Class

    End Class



    '============================================================================================================================



    ''' <summary>
    ''' 裏メニューに関する設定値
    ''' </summary>
    ''' <remarks></remarks>
    Public Class SecretMenuParameter

        ''' <summary>
        ''' [裏メニュー]項目設置場所番号
        ''' </summary>
        ''' <remarks></remarks>
        Public Enum LocateNo As Byte

            ''' <summary>ランチメニュー</summary>
            LaunchMenu

            ''' <summary>システムメニュー</summary>
            SystemMenu

        End Enum


        '::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::


        ''' <summary>
        ''' [裏メニュー]項目の設置場所
        ''' </summary>
        ''' <value></value>
        ''' <returns></returns>
        ''' <remarks></remarks>
        <System.Xml.Serialization.XmlElement("裏メニュー項目設置場所")> _
        Public Property Locate As LocateNo = LocateNo.SystemMenu

        ''' <summary>
        ''' マウスのホイールに関する設定
        ''' </summary>
        ''' <value></value>
        ''' <returns></returns>
        ''' <remarks></remarks>
        <System.Xml.Serialization.XmlElement("マウスホイール")> _
        Public Property MouseWheel As New MouseWheelParameter





        '||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||





        ''' <summary>
        ''' マウスのホイールに関する設定値
        ''' </summary>
        ''' <remarks></remarks>
        Public Class MouseWheelParameter

            ''' <summary>
            ''' 無段ホイールモードを使用する
            ''' </summary>
            ''' <value></value>
            ''' <returns></returns>
            ''' <remarks></remarks>
            <System.Xml.Serialization.XmlElement("無段ホイールモード")> _
            Public Property SlipWheelMode As Boolean = True

            ''' <summary>
            ''' 1ノッチ分の移動量
            ''' </summary>
            ''' <value></value>
            ''' <returns></returns>
            ''' <remarks></remarks>
            <System.Xml.Serialization.XmlElement("ノッチ移動量基準値")> _
            Public Property Delta As Integer = 120

            ''' <summary>
            ''' 回転を分離する閾時間
            ''' </summary>
            ''' <value></value>
            ''' <returns></returns>
            ''' <remarks></remarks>
            <System.Xml.Serialization.XmlElement("回転分離閾時間")> _
            Public Property RollSeparateTime As Integer = 250

            ''' <summary>
            ''' 整合性のチェック
            ''' </summary>
            ''' <returns>致命的な不整合（設定フォームにない項目の場合）</returns>
            ''' <remarks></remarks>
            Public Function AdjustmentErrorCheck() As Boolean

                Dim ret As Boolean = False

                '=== ノッチ移動量基準値（禁則処理：常に正数） ===
                If Delta < 0 Then ret = True

                '=== 回転分離閾時間（禁則処理：0以上） ===
                If RollSeparateTime < 0 Then ret = True

                Return ret

            End Function

        End Class

    End Class

End Class





'||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||





''' <summary>
''' 鍵設定パラメータ
''' </summary>
''' <remarks></remarks>
Public Class KeyParameter

    ''' <summary>
    ''' ダイヤル
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <System.Xml.Serialization.XmlElement("ダイヤル")> _
    Public Property Dial As String = ""

    ''' <summary>
    ''' 合言葉
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <System.Xml.Serialization.XmlElement("合言葉")> _
    Public Property Password As String = ""

End Class

#End Region
